/* puptest - puppeteer with assert */
const puppeteer = require('puppeteer');
const assert = require("assert");
const chai = require('chai');

const chromiumArgs = ["--no-sandbox"];
const chromiumPath = process.env.CHROMIUM_PATH || null;

function newPage() {
  const promise = puppeteer.launch({
    executablePath: chromiumPath,
    args: chromiumArgs,
    ignoreHTTPSErrors: true,
  });

  return promise.then(browser => browser.newPage());
}

class assertPageAction {
  constructor(page, baseUrl) {
    this.page = page;
    this.baseUrl = baseUrl;
  };

  _wrapDoesNotReject(method, message, args) {
    return assert.doesNotReject(method.apply(this.page, args), message);
  }

  url() {
    return this.page.url();
  }

  goto(url, options) {
    return this.page.goto(this.baseUrl + url, options);
  }

  click(selector) {
    return this._wrapDoesNotReject(this.page.click, `click "${selector}"`, arguments);
  }

  waitForSelector(selector) {
    return this._wrapDoesNotReject(this.page.waitForSelector, `wait for "${selector}"`, arguments);
  }

  type(selector, text) {
    return this._wrapDoesNotReject(this.page.type, `type "${text}" to "${selector}"`, arguments);
  }

  clickAndWaitForNavigation(selector, options) {
    return Promise.all([
      this.click(selector, options),
      this.waitForNavigation(options)
    ]);
  }

  waitForNavigation() {
    return this._wrapDoesNotReject(this.page.waitForNavigation, "wait for navigation", arguments);
  }

  $eval(selector) {
    return this.page.$eval.apply(this.page, arguments);
  }

  $evalOk(selector) {
    return this.page.$eval.apply(this.page, arguments).then(
      resp => { chai.assert.isOk(resp, `eval result for "${selector}"`); });
  }

  $evalNotOk(selector) {
    return this.page.$eval.apply(this.page, arguments).then(
      resp => { chai.assert.isNotOk(resp, `eval result for "${selector}"`); });
  }

  $evalMatch(rex, ...args) {
    const selector = args[0];
    return this.page.$eval.apply(this.page, args).then(
      resp => { chai.assert.match(resp, rex, `eval result for "${selector}" matches ${rex}`); });
  }

  $evalEqual(value, ...args) {
    const selector = args[0];
    return this.page.$eval.apply(this.page, args).then(
      resp => { chai.assert.equal(resp, value, `eval result for "${selector}" matches ${value}`); });
  }
}

exports.assertPageAction = assertPageAction;
exports.newPage = newPage;
