const helper = require('../newslash-test-helper');
const url = require('url');
//const assert = require("assert").strict;
const assert = require("chai").assert;

describe('Create Story', function () {

  before(async () => { await helper.init(); });
  after(async () => { await helper.done(); });

  let submissionUrl;
  let editorUrl;

  describe('Submission list', function () {
    this.timeout(5000);
    it('has submission and clickable', async function () {
      await helper.login();

      const p = helper.getPage();
      await p.goto('/admin/submissions');
      const currentUrl = new URL(p.url());
      assert.equal(currentUrl.pathname, '/admin/submissions', 'client has privilege to access the page');

      // get submission url
      await p.waitForSelector('#submissions-manager table.table a[href^="/submission/"]');
      const newUrl = await p.$eval('#submissions-manager table.table a[href^="/submission/"]',
                                   el => el.href);
      submissionUrl = new URL(newUrl).pathname;
      assert.match(submissionUrl, /^\/submission\/\d+/, "valid submission url");
    });
  });


  describe('Submission page', function () {
    this.timeout(5000);
    it('is shown', async function () {
      const p = helper.getPage();
      await p.goto(submissionUrl);

      // get editor url
      const newUrl = await p.$eval('#content-body a[href^="/admin/story/edit"]',
                                   el => el.href);
      u = new URL(newUrl);
      editorUrl = u.pathname + u.search;
      assert.match(editorUrl, /\/admin\/story\/edit\?subid=\d+/, "valid editor url");
    });
  });

  describe('Story editor', function () {
    this.timeout(5000);
    const title = 'ストーリー投稿テスト';
    const dept = '投稿テスト';

    it('is shown', async function () {
      const p = helper.getPage();
      await p.goto(editorUrl);
      await p.waitForSelector('#story-editor input[name="title"]');
    });

    it('receive input and show preview', async function () {
      const p = helper.getPage();
      // change tytle
      await p.click('#story-editor input[name="title"]', {clickCount: 3, delay: 10});
      await p.type('#story-editor input[name="title"]', title);
      await p.click('#story-editor .post-preview button[type="submit"]');

      // check rejection (no dept given)
      await p.waitForSelector('#story-editor .content-editor-wrap .post-error-message', {timeout: 1000});

      // input department and preview
      await p.type('#story-editor input[name="dept"]', dept);
      await p.click('#story-editor .post-preview button[type="submit"]');
      await p.waitForSelector('#story-editor .post-submit', {timeout: 1000});
      await p.$evalEqual(title, '#post-preview header h1 span', el => el.innerHTML.trim());
      await p.$evalEqual(dept, '#post-preview .property span.dept-name', el => el.innerHTML.trim());
    });

    it('post story', async function () {
      const p = helper.getPage();
      await p.click('#story-editor .post-submit button[type="submit"]');
      // await p.waitForSelector('#story-editor .post-submit', {timeout: 1000});
    });
    
    
  });

});
