package Newslash::Command::testdatainsert;
use Mojo::Base 'Newslash::Command';

use Mojo::Util 'getopt';
use Data::Dumper;
use Encode;

has description => 'Insert test data';
has usage => sub { shift->extract_usage };

has experimental => 1;
has danger => 1;

my $sql_dir = "data/SQL/test_data";

sub run {
    my ($self, @args) = @_;
    return if !$self->check_force_option(\@args);

    my $home = $self->app->home;
    $self->app->log->info("testdatainsert: insert test data...");
    if (!$self->_load_and_run_sql($home->child($sql_dir))) {
        $self->app->log->error("testdatainsert: cannot insert data correctly. abort");
        return;
    }
    $self->app->log->info("testdatainsert: done");
}

sub _load_and_run_sql {
    my ($self, $dir) = @_;

    my $db = $self->app->db;
    my $dbh = $db->start_transaction({mysql_multi_statements => 1});

    for my $sql_file ($dir->list->sort->each) {
        next if ($sql_file !~ m/\.sql$/);
        my $sql = decode('utf-8', $sql_file->slurp);
        my $result = $dbh->do($sql);
        if (!$result) {
            $self->app->log->error("testdatainsert: SQL '$sql_file' execute error - $!\n");
            $db->rollback;
            return;
        }
    }

    $db->commit;
    return 1;
}

1;

=encoding utf8

=head1 NAME

Newslash::Command::testdatainsert - newslash administer command (testdatainsert)

=head1 SYNOPSIS

  Usage: APPLICATION testdatainsert [OPTIONS]

      ./newslash_web testdatainsert

    Options:
      -f  if mode is not "test", need this option

=head1 DESCRIPTION

  L<Newslash::Command::testdatainsert> inserts test data to database.

=head1 ATTRIBUTES

L<Newslash::Command::testdatainsert> inherits all attributes from
L<Newslash::Command> and implements the following new ones.

=head2 description

  my $description = $daemon->description;
  $daemon         = $daemon->description('Foo');

Short description of this command, used for the command list.

=head2 usage

  my $usage = $daemon->usage;
  $daemon   = $daemon->usage('Foo');

Usage information for this command, used for the help screen.

=head1 METHODS

L<Newslash::Command::testdatainsert> inherits all methods from
L<Newslash::Command> and implements the following new ones.

=head2 run

  $daemon->run(@ARGV);

Run this command.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.

=cut
