package Newslash::Model::Base;
use Newslash::Model::Cowrapper -base;

use strict;
use warnings;
use utf8;
use feature ':5.10';

use List::Util qw(any);
use DBI;
use Redis;

use Data::Dumper;

sub import {
    my $class = shift;
    return unless my $flag = shift;

    if ($flag eq '-base') {
        $flag = $class;
    } elsif ($flag eq '-strict') {
        $flag = undef;
    }

    if ($flag) {
        my $caller = caller;
        no strict 'refs';
        push @{"${caller}::ISA"}, $flag;
    }

    $_->import for qw(strict warings utf8);
    feature->import(':5.10');
}

sub on_start_up {
    return 1;
}

sub new_instance_of {
    my ($self, $class) = @_;
    if ($class !~ m/::/) {
        $class = "Newslash::Model::" . $class;
    }
    elsif ($class =~ m/^::/) {
        $class = "Newslash::Model" . $class;
    }
    my $new_obj = $class->new($self->{options});
    if ($self->transaction_mode) {
        $new_obj->use_transaction($self->{_tr_dbh});
    }
    return $new_obj;
}

sub logger {
    my $self = shift;

    if ($self->{options}->{logger}) {
        return $self->{options}->{logger};
    }
    # return dummy object
    return {
            debug => sub {},
            error => sub {},
            fatal => sub {},
            info => sub {},
            warn => sub {},
            append => sub {},
            is_level => sub {},
           };
}

sub debug { shift->logger->debug(@_); }
sub warn { shift->logger->warn(@_); }
sub error { shift->logger->error(@_); }
sub fatal { shift->logger->fatal(@_); }
sub info { shift->logger->info(@_); }


1;
