package Newslash::Model::CustomBoxes;
use Newslash::Model::Base -base;

use DateTime;

use constant TABLE_NAME => 'ns_boxes';

sub select {
    my $self = shift;
    my $params = {@_};

    if ($params->{type} eq "thisday") {
        return $self->_thisday($params);
    }
    elsif ($params->{type} eq "mysite") {
        return $self->_mysite($params);
    }
    return;
}


sub _mysite {
    my ($self, $params) = @_;
    my $user = $params->{_user} || {};
    my $my_links = $user->{config}->{ui}->{my_links};;
    $my_links = "" if !defined $my_links;

    return [$my_links];
}

sub _thisday {
    my ($self, $params) = @_;
    my $user = $params->{_user} || {};
    my $offset_sec = $user->{config}->{ui}->{offset_sec} || 0;
    my $stories = $self->new_instance_of("Stories");

    my $now = DateTime->now;
    $now->add(seconds => -$offset_sec) if $offset_sec;

    my @items;
    for (my $year = $now->year; $year > 2000; $year--) {
        my $rs = $stories->select(year => $year,
                                  month => $now->month,
                                  day => $now->day,
                                  offset_sec => $offset_sec,
                                  order_by => { hits => "DESC" },
                                  limit => 1,
                                 );
        if ($rs && @$rs) {
            $rs->[0]->{year} = $year;
            push @items, @$rs;
        }
    }
    my @rs = sort { $b->{hits} <=> $a->{hits} } @items;

    my $limit = $params->{limit} || 8;
    if (@rs > $limit) {
        @rs = @rs[0..$limit];
    }
    my @rs = sort { $b->{year} <=> $a->{year} } @rs;

    return \@rs;
}

1;
