package Newslash::Model::Feeds::Items;
use Newslash::Model::Base -base;
use Data::Dumper;
use DateTime::Format::MySQL;

sub select {
    my $self = shift;
    my $params = {@_};

    my $unique_keys = { id => "ns_feed_items.id",
                        url => "ns_feed_items.url",
                      };
    my $keys = { create_time => "ns_feed_items.create_time",
                 feed_id => "ns_feed_items.feed_id",
                 feed_name => "ns_feeds.name",
                 modified => "ns_feed_items.modified",
               };
    my $timestamp = "ns_feed_items.create_time";

    if (!$params->{order_by}) {
        $params->{order_by} = { modified => "DESC" };
    }

    my ($where_clause, $where_values, $unique) = $self->build_where_clause(unique_keys => $unique_keys,
                                                                           keys => $keys,
                                                                           timestamp => $timestamp,
                                                                           params => $params);
    my ($limit_clause, $limit_values) = $self->build_limit_clause(params => $params);
    my ($orderby_clause, $orderby_values) = $self->build_order_by_clause(keys => $keys,
                                                                         params => $params);
    my @attrs;
    push @attrs, @$where_values, @$limit_values, @$orderby_values;

    my $dbh = $self->connect_db;

    my $sql = "";
    if ($params->{feed_name}) {
        $sql = <<"EOSQL";
SELECT ns_feed_items.* FROM ns_feed_items
 LEFT JOIN ns_feeds ON ns_feeds.feed_id = ns_feed_items.feed_id
    $where_clause
    $orderby_clause
    $limit_clause
EOSQL
    }
    else {
        $sql = <<"EOSQL";
SELECT * FROM ns_feed_items
    $where_clause
    $orderby_clause
    $limit_clause
EOSQL
    }

    #warn($sql);
    #warn(Dumper(@attrs));

    my $sth = $dbh->prepare($sql);
    $sth->execute(@attrs);
    my $items = $sth->fetchall_arrayref({});

    if (!$items) {
        $self->disconnect_db();
        return;
    }
    if (@$items == 0) {
        $self->disconnect_db();
        return $unique ? undef : [];
    }

    return $items->[0] if $unique;
    return $items;
}

sub create {
    my $self = shift;
    my $params = {@_};
    my $url = $params->{url};
    my $feed_id = $params->{feed_id};
    my $title = $params->{title};

    return if (!$url || !$feed_id || !$title);

    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
INSERT INTO ns_feed_items
    (feed_id, url, title, content, content_type, modified, create_time)
  VALUES
    (?,       ?,   ?,     ?,       ?,            ?,        NOW())
  ON DUPLICATE KEY UPDATE
    title = ?, content = ?, content_type = ?, modified = ?
EOSQL

    my $modified;
    if ($params->{modified}) {
        $modified = DateTime::Format::MySQL->format_datetime($params->{modified});
    }

    my $rs = $dbh->do($sql, undef,
                      $feed_id,
                      $url,
                      $params->{title} || "",
                      $params->{content},
                      $params->{content_type},
                      $modified,
                      $params->{title} || "",
                      $params->{content},
                      $params->{content_type},
                      $modified
                     );
    if (!$rs) {
        return;
    }
    my $id = $dbh->last_insert_id(undef, undef, undef, undef);
    if ($id) {
        return $id;
    }
    else {
        return 0;
    }
}

sub update {
    my $self = shift;
    my $params = {@_};
    my $id = $params->{id};
    return if !$id;

    my @cols;
    my @attrs;
    for my $k (qw|url title content content_type|) {
        if (defined $params->{$k}) {
            push @cols, "$k = ?";
            push @attrs, $params->{$k};
        }
    }
    if ($params->{modified}) {
        my $modified = DateTime::Format::MySQL->format_datetime($params->{modified});
        push @cols, "modified = ?";
        push @attrs, $modified;
    }


    my $columns = join(", ", @cols);
    my $sql = <<"EOSQL";
UPDATE ns_feed_items
  SET $columns
  WHERE id = ?
EOSQL

    push @attrs, $id;
    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, @attrs);
    return if !$rs;
    return $id;
}

sub delete {
    my $self = shift;
    my $params = {@_};
    my $id = $params->{id};
    return if !$id;

    my $sql = <<"EOSQL";
DELETE FROM ns_feed_items
  WHERE id = ?
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, $id);
    return $rs;
}

1;
