package Newslash::Model::Loader;

use strict;
use warnings;
use utf8;
use feature ':5.10';

use Module::Load qw(load);
use Mojo::Log;
use Data::Dumper;

sub import {
    my $class = shift;
    my $caller = caller;

    {
        # export functions
        no strict 'refs';
        *{"$caller\::load"} = \&load_model;
        *{"$caller\::loader"} = \&loader;
        *{"$caller\::startup"} = \&startup;
    }

    $_->import for qw(strict warings utf8);
    feature->import(':5.10');
}

state $models;

sub loader {
    my $opts = shift;
    my @sub_opts = @_;
    my $r = $models;

    return sub {
        my $caller = shift;
        my $a = shift || $caller;
        my $c = $r->{$a};

        if ($c) {
            my $ret = $c->new($opts, @sub_opts);
            return $ret;
        }
        return;
    };
}

sub load_model {
    my ($accessor, $class) = @_;
    load $class;
    if (!$models) {
        $models = {};
    }
    $models->{$accessor} = $class;
}

sub startup {
    my $opts = shift;
    my $app = shift;
    my @params = @_;

    my $log = $app->log;
    $log->debug("Model::Loader: startup...");

    return 1 if !$models;
    for my $k (keys %$models) {
        my $m = $models->{$k}->new($opts, @params);
        if (!$m->on_start_up($app)) {
            $log->fatal("$models->{$k}: startup routine faild...");
        }
    }
    return 1;
}

1;

