package Newslash::Model::Networks::Activities;
use Newslash::Model::Base -base;

use constant DEFAULT_LIMIT => 50;

sub select {
    my $self = shift;
    my $params = {@_};

    my $srcid = $params->{src_id} || $params->{srcid};
    my $uid = $params->{user_id} || $params->{uid};
    my $user = $params->{user};

    my $nickname;

    if (!$user && $uid) {
        $user = $self->new_instance_of('Users')->select(uid => $uid);
    }

    if ($user) {
        $nickname = $user->{nickname};
        $uid = $user->{uid};
    }

    my ($coms, $subs, $mods);
    if ($uid) {
        ($coms, $subs, $mods) = $self->_activities_by_uid($uid, $params);
    }
    elsif ($srcid) {
        ($coms, $subs, $mods) = $self->_activities_by_srcid($srcid, $params);
    }

    my $work = {};
    my $item_of = { comment => $coms, submission => $subs, mod => $mods };
    for my $key (qw(comment submission mod)) {
        my $items = $item_of->{$key};
        next if !$items;
        for my $i (@$items) {
            my $u = $i->{uid} || $uid;
            my $nick = $i->{nickname} || $nickname;
            $work->{$u} = {} if !$work->{$u};
            for my $src ($srcid, $i->{ipid}, $i->{subnetid}) {
                next if !$src;
                if (!$work->{$u}->{$src}) {
                    $work->{$u}->{$src} = {
                                           uid => $u,
                                           nickname => $nick,
                                           srcid => $src,
                                           first => $i->{first},
                                           last => $i->{last},
                                           count => $i->{count},
                                           $key => 1,
                                          };
                }
                else {
                    my $h = $work->{$u}->{$src};
                    $h->{first} = $i->{first} if $h->{first} lt $i->{first};
                    $h->{last} = $i->{last} if $h->{last} gt $i->{last};
                    $h->{count} += $i->{count};
                    $h->{$key} = 1;
                }
            }
        }
    }
    my @result;
    for my $i (keys %$work) {
        for my $j (keys %{$work->{$i}}) {
            push @result, $work->{$i}->{$j};
        }
    }

    @result = sort { $b->{last} cmp $a->{last} } @result;
    return \@result;
}

sub _activities_by_uid {
    my ($self, $uid, $params) = @_;
    my ($coms, $subs, $mods);
    $params ||= {};

    my $limit = $params->{limit} || DEFAULT_LIMIT;

    my $dbh = $self->connect_db;

    # select comment activity
    my $sql = <<"EOSQL";
SELECT ipid, subnetid,
    MIN(last_update) AS first,
    MAX(last_update) AS last,
    COUNT(last_update) AS count
  FROM comments
  JOIN users USING(uid)
  WHERE users.uid = ?
  GROUP BY ipid, subnetid
  LIMIT $limit
EOSQL
    my $sth = $dbh->prepare($sql);;
    $sth->execute($uid);
    $coms = $sth->fetchall_arrayref({});

    # select submission activity
    $sql = <<"EOSQL";
SELECT ipid, subnetid,
    MIN(time) AS first,
    MAX(time) AS last,
    COUNT(time) AS count
  FROM submissions
  WHERE uid = ?
  GROUP BY ipid, subnetid
  LIMIT $limit
EOSQL
    $sth = $dbh->prepare($sql);;
    $sth->execute($uid);
    $subs = $sth->fetchall_arrayref({});

    # select mods activity
    $sql = <<"EOSQL";
SELECT ipid, subnetid,
    MIN(ts) AS first,
    MAX(ts) AS last,
    COUNT(ts) AS count
  FROM moderatorlog
  WHERE uid = ?
  GROUP BY ipid, subnetid
  LIMIT $limit
EOSQL
    $sth = $dbh->prepare($sql);;
    $sth->execute($uid);
    $mods = $sth->fetchall_arrayref({});

    $self->disconnect_db;

    return ($coms, $subs, $mods);
}

sub _activities_by_srcid {
    my ($self, $srcid, $params) = @_;
    my ($coms, $subs, $mods);
    $params ||= {};

    my $limit = $params->{limit} || DEFAULT_LIMIT;

    my $dbh = $self->connect_db;

    # select comment activity
    my $sql = <<"EOSQL";
SELECT users.uid, users.nickname,
    MIN(last_update) AS first,
    MAX(last_update) AS last,
    COUNT(last_update) AS count
  FROM comments
  JOIN users USING(uid)
  WHERE ipid = ? OR subnetid = ?
  GROUP BY users.uid, users.nickname
  LIMIT $limit
EOSQL
    my $sth = $dbh->prepare($sql);;
    $sth->execute($srcid, $srcid);
    $coms = $sth->fetchall_arrayref({});

    # select submission activity
    $sql = <<"EOSQL";
SELECT users.uid, users.nickname,
    MIN(time) AS first,
    MAX(time) AS last,
    COUNT(time) AS count
  FROM submissions
  JOIN users USING(uid)
  WHERE ipid = ? OR subnetid = ?
  GROUP BY users.uid, users.nickname
  LIMIT $limit
EOSQL
    $sth = $dbh->prepare($sql);;
    $sth->execute($srcid, $srcid);
    $subs = $sth->fetchall_arrayref({});

    # select mods activity
    $sql = <<"EOSQL";
SELECT users.uid, users.nickname,
    MIN(ts) AS first,
    MAX(ts) AS last,
    COUNT(ts) AS count
  FROM moderatorlog
  JOIN users USING(uid)
  WHERE ipid = ? OR subnetid = ?
  GROUP BY users.uid, users.nickname
  LIMIT $limit
EOSQL
    $sth = $dbh->prepare($sql);;
    $sth->execute($srcid, $srcid);
    $mods = $sth->fetchall_arrayref({});

    $self->disconnect_db;
    return ($coms, $subs, $mods);
}

1;
