package Newslash::Model::Polls::Answers;
use Newslash::Model::Base -base;

sub select {
    my $self = shift;
    my $params = {@_};
    my $qid = $params->{question_id} || $params->{qid} || 0;
    my $aid = $params->{answer_id} || $params->{aid} || 0;
    my $where_clause = "";
    my $limit_clause = "";
    my $order_by_clause = "";
    my @query_param;

    if ($qid) {
        $where_clause = "WHERE qid = ?";
        push @query_param, $qid;
    }
    else {
        $order_by_clause = "ORDER BY date DESC";
        my $offset = $params->{offset} || 0;
        my $limit = $params->{limit} || 10;
        $limit_clause = "LIMIT ?";
        push @query_param, $limit;
        if ($offset) {
            $limit_clause = "$limit_clause OFFSET ?";
        push @query_param, $offset;
        }
    }

    my $sql = <<"EOSQL";
SELECT pollquestions.*, users.nickname AS author
  FROM pollquestions
    LEFT JOIN users ON pollquestions.uid = users.uid
  $where_clause
  $order_by_clause
  $limit_clause
EOSQL


    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute(@query_param);
    my $polls = $sth->fetchall_arrayref(+{});
    if (!$polls) {
        $self->disconnect_db();
        return;
    }
    $self->disconnect_db();

    if (!$qid) {
        for my $poll (@$polls) {
            $self->_generalize($poll);
        }
        return $polls;
    }

    return if @$polls == 0;
    my $poll = $polls->[0];
    my $answers = $self->generic_select(table => "pollanswers",
                                        keys => [qw(qid aid)],
                                        params => {
                                                   qid => $qid,
                                                   order_by => { aid => 'ASC' },
                                                  });
    $self->_calc_percentage($answers);
    return if !$answers;
    $poll->{answers} = $answers;
    $self->_generalize($poll);
    return $poll;
}


1;
