package Newslash::Model::Users::Param;
use Newslash::Model::Base -base;

sub key_definition {
    return {
            table => "users_param",
            primary => "param_id",
            other => [qw(uid name value)],
            aliases => { user_id => "uid",
                         id => "param_id",
                       }
           };
}


sub set {
    my $self = shift;
    my $params = {@_};
    my $uid = $params->{uid};
    my $name = $params->{name};
    my $value = $params->{value};

    if (!defined $uid
        || !defined $name
        || !defined $value) {
        $self->last_error("INVALID_PARAMETER");
        return;
    }

    my $ph = "?";
    if (ref($value) eq "HASH") {
        for my $subkey (keys %$value) {
            if (lc($subkey) eq "function") {
                $ph = $value->{function};
                $value = undef;
                last;
            }
        }
    }

    my @params = defined $value
      ? ($uid, $name, $value, $name, $value)
      : ($uid, $name, $name);

    my $sql = <<"EOSQL";
INSERT INTO users_param
    (uid, name, value)
  VALUES
    (?,   ?,    $ph)
  ON DUPLICATE KEY UPDATE
    name = ?, value = $ph
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, @params);

    if (!defined $rs) {
        $self->last_error($dbh->errstr);
    }
    $self->disconnect_db;
    return $rs;
}

sub insert {
    my $self = shift;
    my $params = {@_};
    my $rs = $self->generic_insert(params => $params);
    return if !defined $rs;

    return $rs;
}

sub select {
    my $self = shift;
    my $params = {@_};
    my $rs = $self->generic_select(params => $params);
    return if !defined $rs;

    my $result = {};
    for my $param (@$rs) {
        $result->{$param->{name}} = $param->{value};
    }
    return $result;
}

sub update {
    my $self = shift;
    my $params = {@_};
    if (!defined $params->{uid} || !defined $params->{name}) {
        $self->set_error("no_uid_and_name_given");
        return;
    }
    my $new_params = { value => $params->{value},
                       where => { uid => $params->{uid},
                                  name => $params->{name} }
                     };
    return $self->generic_update(params => $new_params);
}

sub delete {
    my $self = shift;
    my $params = {@_};
    return $self->generic_delete(params => $params);
}

1;
