package Newslash::Model::Util;
use Newslash::Model::Base -base;

use Data::Dumper;
use Mojo::Log;

use EscapeHTML qw();
use HTML::Tidy;

=head2 tidy_html($html)

tidy HTML

=over 4

=item Parameters

=over 4

=item $html

input HTML

=back

=item Return value

tidied HTML

=back

=cut

sub tidy_html {
    my ($self, $html) = @_;
    my $tidy = HTML::Tidy->new( {
                                 tidy_mark => 0,
                                 doctype => 'omit',
                                 'show-body-only' => 1,
                     } );
    my $tidied = $tidy->clean($html);
    return $tidied;
}

=head2 clean_html(\%allowed, $html)

escape and tidy HTML

=over 4

=item Parameters

=over 4

=item $html

input HTML

=item \%allowed

allowed tag and attribute

=back

=item Return value

tidied HTML

=back

=cut

sub clean_html {
    my ($self, $allowed, $html) = @_;
    my $escaped = $self->escape_html($allowed, $html);
    $escaped = EscapeHTML::blank_line_to_paragraph($escaped);

    my $tidy = HTML::Tidy->new( {
                                 tidy_mark => 0,
                                 doctype => 'omit',
                                 'show-body-only' => 1,
                     } );
    my $tidied = $tidy->clean($escaped);
    chomp($tidied);
    return $tidied;
}


=head2 escape_html(\%allowed, $html)

escape  HTML

=over 4

=item Parameters

=over 4

=item $html

input HTML

=item \%allowed

allowed tag and attribute

=back

=item Return value

escaped HTML

=back

=cut

sub escape_html {
    my ($self, $allowed, $html) = @_;
    return EscapeHTML::escape($allowed, $html);
}


=head2 escape_plaintext($text)

escape  HTML

=over 4

=item Parameters

=over 4

=item $text

input plain text

=back

=item Return value

escaped text

=back

=cut

sub escape_plaintext {
    my ($self, $text) = @_;

    $text =~ s/(?!|lt;|gt;)/&amp;/g;
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;

    return $text;
}

1;
