package Newslash::Plugin::AccessControl;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::Util qw(md5_sum dumper);
use List::Util qw(any);
use Socket;

sub register {
    my ($self, $app, $conf) = @_;

    # set default config value
    my $cnf = $app->config->{ACL} ||= {};
    $cnf->{exclude} ||= ["^/img/", "^/css/", "^/js/", "^/static/", "^/login", "^/logout", "^/banned"];

    $app->hook(around_action => sub {
                   my ($next, $c, $action, $last) = @_;
                   my $user = $c->stash('user');
                   my $conf = $c->app->config || {};
                   my $the_url = $c->url_for;
                   my $acl_conf = $conf->{ACL} || {};

                   if (any { $the_url =~ m/$_/ } @{$acl_conf->{exclude}}) {
                       return $next->();
                   }

                   # check banned
                   if (!$user->{is_admin} && !$user->{permissions}->{access}) {
                       $c->redirect_to('/banned');
                       $c->res->code(307);
                       return;
                   }

                   # check seclev
                   my $page_seclev = $c->stash('seclev') || 0;
                   return $next->() if $page_seclev == 0;

                   my $user_seclev = $user->{seclev} || 0;

                   if ($user_seclev < $page_seclev) {
                       if (!$user->{is_login}) {
                           $c->res->code(307);
                           $c->redirect_to('/login');
                           return;
                       }
                       $c->rendered(403);
                       return;
                   }

                   return $next->();
               });
}


1;
