package Newslash::Plugin::DefaultConfig;
use Mojo::Base 'Mojolicious::Plugin';

my $defaults = {
                Eventd => { log_level => "debug" },

		hypnotoad => { listen => [ 'http://*:80',
					   'https://*:443?cert=/etc/letsencrypt/live/example.com/cert.pem&key=/etc/letsencrypt/live/example.com/privkey.pem'
					 ],
			       pid_file =>  '/var/run/newslash/newslash.pid',
			     },

		Site => { admin_email => 'admin@example.com',
			  name => "newslash",
                          root => "/",
			  base_url => "https://example.com",
			  description => "this is newslash",
			  img_path => "/img",
			  js_path => "/js",
			  css_path => "/css",
			  topic_icon_base_url => "/img/topic_icon",
                          publisher => "example.com",
			},

		Log => { system_log => { mode => "console",
					 local_file => "/var/log/newslash/newslash.log",
					 level => "debug",
				       },
			 backtrace => 0,
		       },

		Search => { item_per_page => 10,
                            searchd_host => "127.0.0.1",
			    searchd_port => 6000,
                            connect_timeout => 2,
                            request_timeout => 5,
			  },

		Polls => { TTL => 120,
			   TTL_unit => "days",
			   vote_cooling_seconds => 300,
			 },

		Profiler => { profiler => "nytprof",
			      enable => 0 },

		FeedFetcher => { fetch_timeout => 5,
				 http_proxy => "",
				 https_proxy => "",
			       },

		System => { secret_key => "<SECRET_KEY>",
			    session_store => "kvs",
			    salt => '',
			    readonly => 0,
			    static_dir => [],
                            session_expiration => 2592000, # about 30 days
			  },

		Repository => {path => "/var/newslash/repository" },

		Analytics => { tag => "" },

		Story => { title_max_byte => 100,
			   submission_karma_bonus => 3,
			 },

		IndexPage => { story_per_page => 10, },

		Timeline => { popular_period => { hours => 6 },
			      item_per_page => 20,
                              item_per_page_limit => 100,
                              enable_tl2 => 0,
                              heatmap => { black  => -999,
                                           violet => -20,
                                           indigo => 25,
                                           blue   => 93,
                                           green  => 138,
                                           yellow => 175,
                                           orange => 200,
                                           red    => 240,
                                         },
			    },

		Database => { host => "db",
			      name => "newslash",
			      user => "newslash",
			      password => "PASSWORD"
			    },

		KeyValueStore => { type => "redis",
				   host => "localhost:6379"
				 },

		Legacy => { memcached => "sd-master:11211" },

		Editor => { allowed_tags => { b =>  [],
					      i =>  [],
					      p =>  [],
					      br => [],
					      a =>  ["href"],
					      ol => ["start"],
					      ul => [],
					      li => [],
					      dl => [],
					      dt => [],
					      dd => [],
					      em => [],
					      strong => [],
					      tt => [],
					      blockquote => ["title", "cite"],
					      div => [],
					      ecode => [],
					      del => [],
					      ins => [],
					      sub => [],
					      sup => [],
					      quote => [],
					      strike => [],
					    },
			  },
                Comments => { allowed_tags => { a => ["href"],
                                                blockquote => [],
                                                i => [],
                                                strong => [] },
                            },
               };

sub register {
    my ($self, $app, $conf) = @_;

    for my $sect_name (keys %$defaults) {
        my $section = $defaults->{$sect_name};
        next if !$section;
        $app->config($sect_name, {}) if !$app->config->{$sect_name};
        my $cfg = $app->config->{$sect_name};
        for my $k (keys %$section) {
            if (!defined $cfg->{$k}) {
                $cfg->{$k} = $section->{$k};
            }
        }
    }
}


1;

=encoding utf8

=head1 NAME

Newslash::Plugin::DefaultConfig - Assign default value to configuration values

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::DefaultConfig');

=head1 DESCRIPTION

L<Newslash::Plugin::DefaultConfig> assing configuration values to $app->config.

=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register DefaultConfig in L<Mojolicious> application.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
