package Newslash::Plugin::ViewFunctions;
use Mojo::Base 'Mojolicious::Plugin';

use strict;
use warnings;
use utf8;
use feature ':5.10';
use Data::Dumper;

sub register {
  my ($self, $app, $conf) = @_;
  $self->{app} = $app;
  $app->helper(view_functions => sub { state $view_functions = $self; });
}

sub format_timestamp {
    my $self = shift;
    my $params = {};

    if (!ref($_[0])) {
        $params = {@_};
    } else {
        $params = shift;
    }

    my $offset_sec = 0;
    if ($params->{user} && $params->{user}->{config}->{offset_sec}) {
        $offset_sec = $params->{user}->{config}->{offset_sec};
    }

    my $dt;
    if ($params->{from_epoch}) {
        $dt = DateTime->from_epoch(epoch => $params->{from_epoch});
    } elsif ($params->{datetime}) {
        $dt = $params->{datetime}
    } else {
        return;
    }

    if ($offset_sec) {
        $dt->add(seconds => $offset_sec);
    }

    my $format = $params->{format} || $params->{user} ? "user" : "simple";
    if ($format eq "simple") {
        return $dt->strftime('%Y-%m-%d %H:%M:%S');
    } elsif ($format eq "user" && $params->{user}) {
        return $dt->strftime($params->{user}->{config}->{time_format});
    }

    return;
}

1;

=encoding utf8

=head1 NAME

Newslash::Plugin::ViewFunctions - Utility functions for MVC's View

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::ViewFunctions');

=head1 DESCRIPTION

L<Newslash::Plugin::ViewFunctions> porvides some utility functions to use
in View templates.

=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register functions in L<Mojolicious> application.

=head2 format_timestamp

  $plugin->format_timestamp(user => $user, epoch => $epoch, format => "user")
  $plugin->format_timestamp(datetime => $dt, format => "simple")

Return formated string.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
