package Newslash::Scripts;
use Mojo::Base -base;
use Mojo::File;
has 'app';

#use Newslash::Web;
use Mojolicious;
use constant CONFIG_FILE => '/etc/newslash/newslash.conf';

sub new {
    my $class = shift;
    my $options = {@_};
    #my $app = new Newslash::Web;
    my $app = new Mojolicious;
    $app->moniker("newslash-web");
    $app->home(Mojo::File->new(_detect_home()));

    _load_config($app);

    if ($options->{mode}) {
        $app->mode($options->{mode});
    }
    my $newobj = bless { app => $app }, $class;
    $newobj->startup;

    return $newobj;
}

sub _load_config {
    my $app = shift;

    # load config file
    # first, check existence of /etc/newslash.conf
    if ($app->mode eq 'production' && -e CONFIG_FILE) {
        $app->plugin('Newslash::Plugin::YAMLConfig', file => CONFIG_FILE);
    }
    else {
        $app->plugin('Newslash::Plugin::YAMLConfig');
    }

    # load default configuration values
    $app->plugin('Newslash::Plugin::DefaultConfig');
}

sub startup {
    my $self = shift;
    warn "startup function not inplemented!";
}

sub _detect_home {

    # Environment variable
    my $home;
    if ($ENV{MOJO_HOME}) {
	$home = $ENV{MOJO_HOME};
    }
    else {
	my $path = $INC{"Newslash/Scripts.pm"};
	my @terms = split(/[\/\\]/, $path);
	while (@terms) {
	    my $last = pop(@terms);
	    if ($last =~ m/^(lib|blib)$/) {
		$home = File::Spec->catdir(@terms);
		last;
	    }
	}
    }
    return $home;
}

1;
