package Newslash::Web::Controller::API::Admin::Filter;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

use constant UPDATABLE_KEYS => qw(name description message status regexp ratio 
                                  threshold limit pathes fields);

sub get {
    my $c = shift;
    my $filters = $c->app->model('filters');
    my $data = $c->req->params->to_hash;
    my $filter_id = $data->{id} || $data->{filter_id};

    # filter_id is not given, return all items
    if (!$filter_id) {
        my $items = $filters->select(order_by => {filter_id => "ASC"});
        if (!$items) {
            $c->rendered(500);
            return;
        }
        $c->render(json => {items => $items});
        return;
    }

    # return selected item
    my $item = $filters->select(filter_id => $filter_id);
    if (!$item) {
        $c->rendered(404);
        return;
    }

    $c->render(json => {item => $item});
    return;
}

sub post {
    my $c = shift;
    my $data = $c->req->json;
    my $action = $data->{action} || "";

    if ($action eq "create") {
        return $c->_create;
    }
    elsif ($action eq "update") {
        return $c->_update;
    }
    elsif ($action eq "delete") {
        return $c->_delete;
    }

    $c->render(json => {error => 1, message => "no_action_given"});
    $c->rendered(400);
    return;
}

sub _create {
    my $c = shift;
    my $filters = $c->app->model('filters');
    my $data = $c->req->json;

    my $name = $data->{name};

    my $message = "";
    $message = "no_name_given" if !$name;

    if ($message) {
        $c->render(json => {error => 1, message => $message});
        $c->rendered(400);
        return;
    }

    my $rs = $filters->select(name => $name);
    if ($rs) {
        $c->render(json => {error => 1, message => "name_is_already_used"});
        $c->rendered(400);
        return;
    }

    $data->{pathes} ||= "";
    $data->{fields} ||= "";
    my @params;
    for my $k (UPDATABLE_KEYS) {
        if (defined $data->{$k}) {
            push @params, $k, $data->{$k};
        }
    }

    my $filter_id = $filters->create(@params);
    if (!$filter_id) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {filter_id => $filter_id});
    return;
}

sub _update {
    my $c = shift;
    my $filters = $c->app->model('filters');
    my $data = $c->req->json;
    my $filter_id = $data->{id} || $data->{filter_id};

    if (!$filter_id) {
        $c->rendered(400);
        return;
    }

    my $rs = $filters->select(filter_id => $filter_id);
    if (!$rs) {
        $c->render(json => {error => 1, message => "not_found"});
        $c->rendered(400);
        return;
    }

    $data->{pathes} ||= "";
    $data->{fields} ||= "";
    my @params;
    for my $k (UPDATABLE_KEYS) {
        if (defined $data->{$k}) {
            push @params, $k, $data->{$k};
        }
    }

    $rs = $filters->update(filter_id => $filter_id, @params);
    if (!$rs) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {filter_id => $filter_id});
    return;
}

sub _delete {
    my $c = shift;
    my $filters = $c->app->model('filters');
    my $data = $c->req->json;
    my $filter_id = $data->{id} || $data->{filter_id};

    if (!$filter_id) {
        $c->rendered(400);
        return;
    }

    my $rs = $filters->select(filter_id => $filter_id);
    if (!$rs) {
        $c->render(json => {error => 1, message => "not_found"});
        $c->rendered(400);
        return;
    }

    $rs = $filters->delete(filter_id => $filter_id);

    if (!$rs) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {error => 0});
    return;
}

1;

