package Newslash::Web::Controller::API::Admin::Repository;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

use File::Spec;
use List::Util qw(any);

sub export {
    my $c = shift;
    my $param = $c->req->params->to_hash;

    my $target = $param->{target};
    my $message;

    # check parameters
    $message = "no_target_given" if !$target;

    my $repo_path = $c->app->config->{Repository}->{path};
    $message = "repository_not_configured" if !$repo_path;

    if ($message) {
        $c->render(json => {error => 1, message => $target});
        $c->rendered(400);
        return;
    }

    my $model_name;
    if (any { $target eq $_ } qw(feeds boxes filters) ) {
        $model_name = $target;
    }
    else {
        $c->render(json => { error => 1, message => "invalid_target" });
        $c->rendered(400);
        return;
    }
    my $file_name = "$model_name.json";

    my $file_path = File::Spec->catfile($repo_path, $file_name);
    my $rs = $c->app->model($model_name)->export_all($file_path);
    if (!$rs) {
        $c->render(json => { error => 1, message => "export_failed" });
        $c->rendered(500);
        return;
    }
    $c->render(json => {});
    return;
}

sub import {
    my $c = shift;
    my $param = $c->req->params->to_hash;

    my $target = $param->{target};
    my $message;

    # check parameters
    $message = "no_target_given" if !$target;

    my $repo_path = $c->app->config->{Repository}->{path};
    $message = "repository_not_configured" if !$repo_path;

    if ($message) {
        $c->render(json => {error => 1, message => $target});
        $c->rendered(400);
        return;
    }

    my $model_name;
    if (any { $target eq $_ } qw(feeds boxes filters) ) {
        $model_name = $target;
    }
    else {
        $c->render(json => { error => 1, message => "invalid_target" });
        $c->rendered(400);
        return;
    }
    my $file_name = "$model_name.json";

    my $file_path = File::Spec->catfile($repo_path, $file_name);
    my $rs = $c->app->model($model_name)->import_all($file_path);
    if (!$rs) {
        $c->render(json => { error => 1, message => "import_failed" });
        $c->rendered(500);
        return;
    }
    $c->render(json => {});
    return;
}

1;

