package Newslash::Web::Controller::API::Comment;
#use Mojo::Base 'Mojolicious::Controller';
use Mojo::Base 'Newslash::Web::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json);
use Mojo::Date;

sub post {
    my $c = shift;
    my $params = {};
    my $extra_params = {};
    my $opts = {};
    my $user = $c->stash('user');
    my $users = $c->model('users');

    # check permission
    if (!$user->{permissions}->{comment}) {
        $c->render(json => { err => 1, message => "not_allowed" });
        $c->rendered(403);
        return;
    }

    my $comments = $c->model('comments');
    my $discussions = $c->model('discussions');

    my $conf = $c->config->{Comments} || {};
    my $data = $c->req->json;
    my $message = "";

    $params->{title} = $c->format_htmltext($data->{title}, "title");
    $params->{comment} = $c->format_htmltext($data->{comment}, "comment");
    $params->{discussion_id} = $data->{discussion_id};
    $params->{stoid} = $data->{stoid};
    $params->{pid} = $data->{pid};
    $params->{action} = $data->{action} || 'preview';

    $message = "no title!" if !$params->{title};
    $message = "no comment!" if !$params->{comment};

    my $discussion = $discussions->select(discussion_id => $params->{discussion_id});
    #$c->apply_seclev_filter("discussions", $discussion);

    if (!$discussion) {
        $message = "invalid_discussion";
    }
    else {
        $message = "discussion_is_archived" if $discussion->{type} eq 'archived';
    }

    if (!$discussions->is_comment_allowed(discussion => $discussion,
                                          uid => $user->{uid})) {
        $message = "not_allowed";
    }


    if (length($message) > 0) {
        $c->render(json => { err => 1, message => $message });
        $c->rendered(400);
        return;
    }

    $data->{title} = $params->{title};
    $data->{comment} = $params->{comment};
    $data->{author} = $user->{nickname};
    $data->{signature} = $user->{sig};
    $data->{date} = Mojo::Date->new->to_datetime;

    if ($data->{action} eq 'preview') {
        $c->render(json => $data);
        return;
    }
    elsif ($params->{action} eq 'post') {
        my $cid = $comments->create($params, $user, $extra_params, $opts);
        if ($cid) {
            my $ret = {};
            my $result = $comments->select(cid => $cid,
                                           security_level => $user->{seclev});
            $c->apply_seclev_filter("comment", $result);

            if ($result) {
                $ret->{type} = "comment";
                $ret->{id} = $cid;
                $ret->{cid} = $cid;
                $ret->{new_comment} = $result;
                $c->render(json => $ret);
                $c->event_que->emit("comment", "post", $user->{uid}, $cid);
                $c->stats->add_event_counter("comment_post");
            }
            else {
                $c->render(json => { error => 1, message => "cannot get created comment" });
                $c->rendered(500);
            }
        }
        else {
            warn $comments->last_error . ": " . dumper($params);
            if ($comments->last_errorno && $comments->last_errorno == 1062) { #ER_DUP_ENTRY
                $c->rendered(409);
                return;
            }
            $c->render(json => { error => 1, message => $comments->last_error });
            $c->rendered(500);
        }
    }
}

sub get {
    my $c = shift;

    my $users = $c->model('users');
    my $user = $c->stash('user');

    my $data = $c->req->query_params->to_hash;
    my ($key, $value);

    if ($data->{pid} || $data->{parent_id}) {
        $key = 'pid';
        $value = $data->{pid} || $data->{parent_id};
    }
    elsif ($data->{discussion_id}) {
        $key = 'discussion_id';
        $value = $data->{discussion_id};
    }
    elsif ($data->{cid} || $data->{comment_id} || $data->{id}) {
        $key = 'cid';
        $value = $data->{cid} || $data->{comment_id} || $data->{id};
    }

    my $comments;
    if ($user->{is_login}) {
        $comments = $c->model('comments');
    }
    else {
        $comments = $c->ccache->model('comments');
    }
    my $rs = $comments->select($key => $value);

    $c->apply_seclev_filter("comment", $rs);

    if (!defined $rs) {
        $c->render(json => { error => 1, message => 'invalid request' });
        $c->rendered(400);
        return;
    }
    if (ref($rs) ne "ARRAY") {
        $rs = [$rs];
    }
    $c->render(json => { $key => $value, comments => $rs });
    return;
}

1;
