package Newslash::Web::Controller::API::Moderation;
use Mojo::Base 'Newslash::Web::Controller';
#use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json to_json from_json);

sub get {
    my $c = shift;
    my $user = $c->stash('user');
    my $moderations = $c->model('moderations');

    my $message = "";
    my $cid = $c->req->param("cid");

    if (!$cid) {
        $message = "valid cid is required";
    }
    else {
        #my $mods = $moderations->select(cid => $cid, active => 1);
        #my $mods = $moderations->select_with_m2($cid, $user->{uid});
        my $mods = $moderations->select(cid => $cid,
                                        uid => $user->{uid},
                                        m2 => 1);
        $c->apply_seclev_filter("moderations", $mods);

        if ($mods) {
            # remove ipid and subnetid
            if (!$user->{is_admin}) {
                for my $mod (@$mods) {
                    delete $mod->{ipid};
                    delete $mod->{subnetid};
                }
            };
            $c->render(json => {cid => $cid,
                                moderations => $mods});
            return;
        }
        $message = "moderations cannot retrive";
    }

    if (length($message) > 0) {
        $c->render(json => { error => 1, message => $message });
        $c->rendered(400);
        return;
    }
    return;
}

sub post {
    my $c = shift;
    my $user = $c->stash('user');
    my $moderations = $c->model('moderations');

    my $data = $c->req->json;
    my $message = "";

    if (!$user->{is_login}) {
        $c->render(json => { error => 1, message => "you are not a valid user." });
        $c->rendered(400);
        return;
    }

    # check permission
    if (!$user->{permissions}->{m1}) {
        $c->render(json => { err => 1, message => "not_allowed" });
        $c->rendered(403);
        return;
    }

    #$c->app->log->debug(dumper($data));
    if ($data->{action} eq "moderate") {
        my $cid = $data->{cid};
        my $reason_id = $data->{reason};
        my $discussion_id = $data->{discussion_id};

        if ($cid && defined $reason_id && $discussion_id) {
            my $mod_id = $moderations->create($user, $cid, $reason_id);
            if ($mod_id) {
                $c->render(json => {message => "moderation accepted"});
                $c->event_que->emit("moderation", "post", $user->{uid}, $mod_id);
                $c->stats->add_event_counter("moderation");
                return;
            }
            $c->render(json => { error => 1, message => $moderations->last_error });
            $c->rendered(400);
            return;
        }
    }

    $message = "invalid request";
    if (length($message) > 0) {
        $c->render(json => { error => 1, message => $message });
        $c->rendered(400);
        return;
    }
    return;
}

1;
