package Newslash::Web::Controller::API::Poll;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json);
use DateTime;
use DateTime::Format::MySQL;

use Newslash::Web::Controller::Poll;

sub _is_poll_active {
    my ($c, $poll) = @_;
    return Newslash::Web::Controller::Poll::_is_poll_active($c, $poll);
}

sub get {
    my $c = shift;
    my $qid = $c->param('qid');
    my $user = $c->stash('user');
    my $polls = $c->model('polls');

    my $poll = $polls->select(qid => $qid);
    if (!$poll) {
        $c->rendered(404);
        return;
    }
    my $discussion_id = $poll->{discussion_id};
    $poll->{active} = _is_poll_active($c, $poll);

    $c->render(json => { item => $poll });
}

sub post {
    my $c = shift;
    my $polls = $c->model('polls');
    my $user = $c->stash('user');

    if (!$user->{is_admin} && !$user->{editor}) {
        $c->render(json => { err => 1, message => "no_permission" });
        $c->rendered(403);
        return;
    }

    my $poll = $c->req->json->{poll} || {};
    my $answers = $poll->{answers};
    my $question = $poll->{question};

    if (!$answers || !$question) {
        $c->render(json => { err => 1, message => "create poll failed" });
        $c->rendered(400);
        return;
    }

    my $qid = $polls->create($user, {
                                     question => $question,
                                     tid => 736,
                                     answers => $answers,
                                    });
    if (!$qid) {
        $c->render(json => { err => 1, message => "create poll failed" });
        $c->rendered(500);
        return;
    }
    $c->render(json => { type => "poll", id => $qid });
    $c->event_que->emit("poll", "post", $user->{uid}, $qid);
    $c->stats->add_event_counter("poll_create");

}

sub vote {
    my $c = shift;
    my $polls = $c->model('polls');
    my $user = $c->stash('user');

    my $vote = $c->req->json->{vote} || {};
    my $qid = $vote->{qid} || 0;
    my $answer = $vote->{aid};

    my $message = "";

    if ($qid !~ m/\A[0-9]+\z/) {
        $qid = 0;
    }
    if (!$qid) {
        $message = "invalid_qid";
    }

    my $poll = $polls->select(qid => $qid);
    if (!$poll) {
        $message = "poll_not_found";
    }
    if (!_is_poll_active($c, $poll)) {
        $message = "vote_is_closed";
    }

    if ($message) {
        $c->render(json => { error => 1, message => $message });
        $c->rendered(400);
        return;
    }

    if (!Newslash::Web::Controller::Poll::_is_valid_voter($c, $user, $qid)) {
        $c->render(json => { type => "answer", id => $answer });
        return;
    }

    my $rs = $polls->vote($user,  $qid, $answer);
    if (!$rs) {
        $message = $polls->last_error;
    }

    if ($message) {
        $c->render(json => { error => 1, message => $message });
        $c->rendered(400);
        return;
    }
    $c->render(json => { type => "answer", id => $answer });
    $c->event_que->emit("poll", "vote", $user->{uid}, $qid);
    $c->stats->add_event_counter("poll_vote");
}


1;
