package Newslash::Web::Controller::API::TagModeration;
use Mojo::Base 'Newslash::Web::Controller';
use List::Util qw(all);

sub post {
    my $c = shift;
    my $user = $c->stash('user');
    my $data = $c->req->json;
    my $message = "invalid request";

    if (!$user->{is_login}) {
        $c->render(json => { error => 1, message => "you are not a valid user." });
        $c->rendered(400);
        return;
    }

    my $tags = $c->model('tags');
    my $params = $data->{params};
    my $tagname = $params->{tagname} || $params->{name};
    my $globj_id = $params->{globjid} || $params->{globj_id};

    if (!$globj_id) {
        $c->render(json => { error => { message => "invalid_globj_id" } });
        $c->rendered(400);
        return;
    }

    if (all { $_ ne $tagname } (qw{nix nod})) {
        $c->render(json => { error => { message => "invalid_modtag" } });
        $c->rendered(400);
        return;
    }

    if ($data->{action} eq "vote") {
        # first, unset nix/nod tag
        my $del_tag = ($tagname eq "nod") ? "nix" : "nod";

        $tags->start_transaction;
        my $rs = $tags->del(uid => $user->{uid},
                            name => $del_tag,
                            globj_id => $globj_id,
                            private => 1
                           );
        if (!$rs) {
            $c->render(json => { error => { message => $tags->last_error } });
            $c->rendered(400);
            $tags->rollback;
            return;
        }
        $rs = $tags->add(uid => $user->{uid},
                         name => $tagname,
                         globj_id => $globj_id,
                         private => 1
                        );
        if (!$rs) {
            $c->render(json => { error => { message => $tags->last_error } });
            $c->rendered(400);
            return;
        }
        $tags->commit;
        $c->render(json => { result => { tag_id => $rs } });
        return;
    }

    if ($data->{action} eq "set") {
        my $rs = $tags->add(uid => $user->{uid},
                         name => $tagname,
                         globj_id => $globj_id,
                         private => 1
                        );
        if (!$rs) {
            $c->render(json => { error => { message => $tags->last_error } });
            $c->rendered(400);
            return;
        }
        $c->render(json => { result => { tag_id => $rs } });
        return;
    }

    if ($data->{action} eq "unset") {
        my $params = $data->{params};
        my $rs = $tags->del(uid => $user->{uid},
                            name => $tagname,
                            globj_id => $globj_id,
                           );
        if (!$rs) {
            $c->render(json => { error => { message => $tags->last_error } });
            $c->rendered(400);
            return;
        }
        $c->render(json => { result => { delete_count => $rs } });
        return;
    }

    if (length($message) > 0) {
        $c->render(json => { error => 1, message => $message });
        $c->rendered(400);
        return;
    }
    return;
}

1;
