package Newslash::Web::Controller::API::Post;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json);

sub submission {
    my $c = shift;
    my $params = {};

    my $user = $c->stash('user');
    my $submissions = $c->model('submissions');

    my $allowed_tags = <<'EOT';
{
  "a": ["href"],
  "blockquote": [],
  "i": [],
  "strong": []
}
EOT

    my $allowed = decode_json($allowed_tags);
    my $data = $c->req->json;

    $params->{title} = $c->format_htmltext($data->{title}, "title");
    $params->{url} = $data->{url};
    $params->{email} = $data->{email};
    $params->{introtext} = $c->clean_html($allowed, $data->{introtext});
    $params->{tid} = 49; #mainpage
    $params->{action} = $data->{action} || 'preview';
    $extra_params->{createheaders} = $c->req->headers->to_string;

    if ($params->{action} eq 'preview') {
        my $result = {
                      title => $params->{title},
                      intro_text => $params->{introtext},
                      email => $params->{email},
                      tid => $params->{tid},
                     };
        $c->render(json => $result);
    }
    elsif ($params->{action} eq 'post') {
        my $sub_id = $submissions->create(user => $user, $params);
        if ($sub_id) {
            $c->render(json => {type => "submission", id => $sub_id});
        }
        else {
            if ($submissions->last_errorno && $submissions->last_errorno == 1062) { #ER_DUP_ENTRY
                $c->rendered(409);
                return;
            }
            $c->rendered(500);
        }
    }
}

sub story {
    my $c = shift;
    my $params = {};

    my $user = $c->stash('user');
    my $stories = $c->model('stories');
    my $users = $c->model('users');

    my $allowed_tags = <<'EOT';
{
  "a": ["href"],
  "blockquote": [],
  "i": [],
  "strong": []
}
EOT

    my $allowed = decode_json($allowed_tags);
    my $data = $c->req->json;
    my $message = "";

    $params->{title} = $c->format_htmltext($data->{title}, "title");
    $message = "no title!" if !$params->{title};

    $params->{time} = $data->{createtime};

    # check author is valid
    my $author = $users->select(nickname => $data->{author});
    if (!defined $author || !$author->{is_admin}) {
        $message = "invalid author";
    }
    $params->{uid} = $author->{uid};

    $params->{dept} = $data->{dept};
    $params->{introtext} = $c->clean_html($allowed, $data->{introtext});
    $params->{bodytext} = $c->clean_html($allowed, $data->{bodytext});
    #$data->{add_related};
    #$data->{mediaurl};
    #$data->{mediatype};
    $params->{commentstatus} = $data->{commentstatus};
    $params->{neverdisplay} = $data->{display} ? 0 : 1;

    $params->{action} = $data->{action} || 'preview';

    # TODO: can select topics
    $params->{topics_chosen} = { 49 => 10, }; # tid => weight

    if (length($message) > 0) {
        $c->render(json => { err => 1, message => $message });
        $c->rendered(400);
        return;
    }

    if ($data->{action} eq 'preview') {
        $data->{introtext} = $params->{introtext};
        $data->{bodytext} = $params->{bodytext};
        $data->{title} = $params->{title};
        $c->render(json => $data);
        return;
    }
    elsif ($params->{action} eq 'post') {
        my $sid = $stories->create(user => $user, %$params);
        if ($sid) {
            $c->render(json => {type => "story", id => $sid});
        }
        else {
            if ($stories->last_errorno && $stories->last_errorno == 1062) { #ER_DUP_ENTRY
                $c->rendered(409);
                return;
            }
            $c->render(json => { err => 1, message => $stories->last_error });
            $c->rendered(500);
        }
    }
}

sub comment {
    my $c = shift;
    my $params = {};
    my $extra_params = {};
    my $opts = {};

    my $users = $c->model('users');
    my $comments = $c->model('comments');
    my $user = $c->stash('user');

    my $allowed_tags = <<'EOT';
{
  "a": ["href"],
  "blockquote": [],
  "i": [],
  "strong": []
}
EOT

    my $allowed = decode_json($allowed_tags);
    my $data = $c->req->json;
    my $message = "";

    $params->{title} = $c->format_htmltext($data->{title}, "title");
    $params->{comment} = $c->clean_html($allowed, $data->{comment});
    $params->{sid} = $data->{discussion_id};
    $params->{stoid} = $data->{stoid};
    $params->{pid} = $data->{pid};
    $params->{action} = $data->{action} || 'preview';

    $message = "no title!" if !$params->{title};
    $message = "no comment!" if !$params->{comment};
    if (length($message) > 0) {
        $c->render(json => { err => 1, message => $message });
        $c->rendered(400);
        return;
    }

    if ($data->{action} eq 'preview') {
        $data->{title} = $params->{title};
        $data->{comment} = $params->{comment};
        $c->render(json => $data);
        return;
    }
    elsif ($params->{action} eq 'post') {
        my $cid = $comments->create($params, $user, $extra_params, $opts);
        if ($cid) {
            $c->render(json => {type => "comment", id => $cid});
        }
        else {
            if ($comments->last_errorno && $comments->last_errorno == 1062) { #ER_DUP_ENTRY
                $c->rendered(409);
                return;
            }
            $c->render(json => { err => 1, message => $comments->last_error });
            warn $comments->last_error;
            $c->rendered(500);
        }
    }
}

1;
