package Newslash::Web::Controller::Admin::Story;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

sub edit {
    my $c = shift;
    my $users = $c->model('users');
    my $authors = $users->select(author => 1);
    my $stories = $c->model('stories');

    my $sub_id = $c->param('subid');
    my $stoid = $c->param('stoid');
    my $source = {};
    my $page = { content_type => "story",
                 type => "edit" };

    if ($stoid) {
        my $story = $c->model('stories')->select(stoid => $stoid);
        if ($story) {
            $source->{type} = "story";
            $source->{id} = $stoid;
            $source->{item} = $story;
            $page->{type} = "story";
            $page->{stoid} = $stoid;
        }
    }
    elsif ($sub_id) {
        my $sub = $c->model('submissions')->select(submission_id => $sub_id);
        if ($sub) {
            $source->{type} = "submission";
            $source->{id} = $sub_id;
            $source->{item} = $sub;
        }
    }

    my $items = $stories->select(limit => 10,
				 order_by => { create_time => "DESC" });

    $c->render(authors => $authors,
	       source => $source,
	       page => $page,
	       recent_stories => $items,
	      );
}

sub list {
    my $c = shift;
    my $page = $c->param('page') || 0;
    my $users = $c->model('users');
    my $stories = $c->model('stories');

    my $items_per_page = 20;
    
    my $items = $stories->select(limit => $items_per_page,
				 offset => $page * $items_per_page,
				 order_by => { create_time => "DESC" });

    # convert submitter UID to user hash
    my $user_cache = {};
    for my $item (@$items) {
	my $submitter = $item->{submitter};
	if (!defined $user_cache->{$submitter}) {
	    my $u = $users->select(uid => $submitter);
	    $user_cache->{$submitter} = $u ? $u : {};
	}
	$item->{submitter} = $user_cache->{$submitter};
    }

    $c->render(items => $items, page_num => $page, );
}

1;

