package Newslash::Web::Controller::Login;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(b64_encode dumper);
use Mojo::JSON qw(to_json);
use DateTime;
use DateTime::Format::MySQL;
use Email::Valid;

sub login {
    my $c = shift;
    my $nickname = $c->param('nickname') || '';
    my $passwd = $c->param('passwd') || '';
    my $users = $c->model('users');
    my $rpath = $c->param('rpath') || '/';

    my $page = { type => "system",
                 content_type => "login",
               };

    # rpath must be absolute path.
    if ($rpath !~ m/^\//) {
        $rpath = '/';
    }

    if (length($nickname) == 0) {
        $c->render(msg => 'SRAD login',
                   rpath => $rpath,
                   page => $page,);
        return;
    }

    # TODO: if user already logged-in, force logout?
    my $user = $users->authentification($nickname, $passwd);
    if ($user) {
        if (!$c->user_auth->do_login($user, $c)) {
            $c->rendered(500);
            return;
        }
        $c->redirect_to($rpath);
        $c->event_que->emit("user", "login", $user->{uid});
        return;
    }

    $c->render(msg => 'Wrong username or password.',
               rpath => $rpath,
               nickname => $nickname,
               page => $page,
              );
}

sub logout {
  my $c = shift;
  my $session = $c->session('session');
  my $user = $c->stash('user');
  $c->user_auth->clear_session($user, $session->{token});

  my $rpath = $c->param('rpath') || '/';

  #TODO: $c->model('users')->expire_token($session->{token});
  $c->session(expires => 1);
  $c->event_que->emit("user", "logout", $c->stash('user')->{uid});
  $c->stats->add_event_counter("user_logout");
  $c->redirect_to($rpath);
}

sub activation {
    my $c = shift;
    my $users = $c->model('users');
    my $nickname = $c->param("nickname");
    my $token = $c->param("token");
    my $cancel = $c->param("cancel");

    my $the_user = $users->select(nickname => $nickname);
    my $prefs = { nickname => $nickname, token => $token };

    # check if token is correct
    if (!$the_user || !$token) {
        $c->render(activation_succeed => 0, error => "INVALID_TOKEN");
        $c->res->code(400);
        return;
    }

    # check if token is valid
    my $rs = $c->users->check_activation_token($nickname, $token);
    if (!$rs) {
        $c->render(activation_succeed => 0, error => $c->users->last_error);
        $c->res->code(400);
        return;
    }

    # if reset activation
    if ($cancel) {
        my $rs = $c->users->cancel_activation($the_user);
        if (!$rs) {
            $c->render(activation_succeed => 0, error => $c->users->last_error || 1);
            $c->res->code(400);
            return;
        }
        $c->render(activation_succeed => 0, error => "CANCEL_ACTIVATION");
        return;
    }

    # ok
    $c->render(prefs => $prefs, activation_succeed => 1, error => "");
    return;
}

sub reset_password {
    my $c = shift;

    if ($c->req->method eq "GET") {
        $c->render();
        return;
    }

    if ($c->req->method eq "POST") {
        my $users = $c->model('users');
        my $email = $c->param("email");

        if (!$email || !Email::Valid->address($email)) {
            $c->render(result => "INVALID_EMAIL", email => $email);
            $c->res->code(400);
            return;
        }

        # get user account from email
        my $the_user = $users->select(realemail => $email);
        if (!$the_user) {
            $c->render(result => "USER_NOT_FOUND", email => $email);
            $c->res->code(400);
            return;
        }

        # check done
        my $rs = $c->users->reset_password($the_user);
        if (!$rs) {
            $c->render(result => $c->users->last_error, email => $email);
            $c->res->code(400);
            return;
        }
        $c->render(result => "DONE", email => $email);
        return;
    }
}

sub newuser {
    my $c = shift;
    my $user = $c->stash('user');

    if ($user->{is_login}) {
        $c->res->code(303);
        $c->redirect_to("/my/");
        return;
    }

    if ($c->req->method eq "GET") {
        $c->render();
        return;
    }

    if ($c->req->method ne "POST") {
        $c->res->code(400);
        return;
    }

    my $prefs = {};
    for my $k (qw(unickname email email2 allow_newsletter allow_headline)) {
        $prefs->{$k} = $c->param($k);
    }

    my $error = 0;
    if (!$prefs->{nickname}) {
        $error = 1;
        $prefs->{id_error} = "BLANK_ID";
    }

    if (!$prefs->{email}) {
        $error = 1;
        $prefs->{mail_error} = "BLANK_MAIL";
    }

    # mail is valid?

    if ($prefs->{email} ne $prefs->{email2}) {
        $error = 1;
        $prefs->{mail_error} = "MAIL_NOT_MATCH";
    }

    if ($error) {
        $c->render(prefs => $prefs);
        return;
    }

    # check ID

    #my $result = $c->model('users')->create_user($c->param('nickname'), $c->param('email'), {});
    # TODO: set newsletter and headline settings
    # if (!$result) {
    #     die "database error!";
    # }
    # if ($result->{error}) {
    #     $c->render(error => 1);
    # } else {
    #     $c->render("login/register_done", uid => $result->{uid}, password => $result->{password});
    # }
}

1;

