package Newslash::Web::Controller::Redirect;
use Mojo::Base 'Mojolicious::Controller';
use Data::Dumper;
use List::Util qw(any);

my @sections = qw(apple askslashdot azure developers
		  hardware idle interview it linux
		  mainpage mobile opensource review
		  science security slash srad yro);

sub _redirect_by_id {
    my $c = shift;
    my $firehose = $c->model('firehose');

    my $fh_item = $firehose->select(id => $c->param('id'));
    if (!$fh_item) {
	$c->render('Not found', status => '404');
	return;
    }

    my $type = $fh_item->{type};
    my $src_id = $fh_item->{srcid};
    if ($type eq "story") {
	my $item = $c->model('stories')->select(id => $src_id);
	if ($item) {
	    $c->res->code(301);
	    $c->redirect_to("/story/$item->{sid}");
	    return;
	}
    } elsif ($type eq "submission") {
	my $item = $c->model('submissions')->select(id => $src_id);
	if ($item) {
	    $c->res->code(301);
	    $c->redirect_to("/submission/$item->{subid}");
	    return;
	}
    } elsif ($type eq "journal") {
	my $item = $c->model('journals')->select(id => $src_id);
	if ($item) {
	    $c->res->code(301);
	    $c->redirect_to("/~$item->{author}/journal/$item->{id}");
	    return;
	}
    } elsif ($type eq "comment") {
	my $item = $c->model('comments')->select(id => $src_id);
	if ($item) {
	    $c->res->code(301);
	    $c->redirect_to("/comment/$item->{cid}");
	    return;
	}
    }
    $c->render('Not found', status => '404');
    return;
}

sub _redirect_by_view {
    my $c = shift;
    my $firehose = $c->model('firehose');
    my $view = $c->param('view');

    if ($c->param('fhfilter')) {
	my $filter = $c->param('fhfilter');

	if ($view eq 'stories') {
	    # check tag
	    my $tag = $c->model('tags')->select_tagnames(tagname => $filter);
	    if ($tag) {
		$c->res->code(301);
		$c->redirect_to("/tag/$filter");
		return;
	    }
	}

	# redirecct to search page
	my $targets = { stories => "story",
			submissions => "submission",
			comments => "comment",
			polls => "poll",
			users => "user",
		      };
	if ($targets->{$view}) {
	    $c->res->code(301);
	    $c->redirect_to("/search?q=$filter&sort_reverse=1&target=$targets->{$view}");
	    return;
	}
    }

    if ($view eq 'stories') {
	if ($c->param('startdate')
	    && $c->param('startdate') =~ m/(\d{2})(\d{2})(\d{2})(\d{2})/) {
	    $c->res->code(301);
	    $c->redirect_to("/story/$2/$3/$4");
	    return;
	}
	$c->res->code(301);
	$c->redirect_to("/");
	return;
    }

    if (any { $view eq $_ } qw(journals submissions comments polls recent popular)) {
	$c->res->code(301);
	$c->redirect_to("/$view/");
	return;
    }
    $c->render('Not found', status => '404');
    return;
}

sub users {
    my $c = shift;
    my $users = $c->model('users');

    my $user;
    if ($c->param('nick')) {
	$user = $users->select(nickname => $c->param('nick'));
    }
    elsif ($c->param('uid')) {
	$user = $users->select(uid => $c->param('uid'));
    }

    if ($user && $user->{nickname}) {
	$c->res->code(301);
	$c->redirect_to("/~$user->{nickname}/");
	return;
    }
    $c->render('Not found', status => '404');
    return;
}

sub index {
    my $c = shift;
    my $url = $c->req->url;
    my $query = $url->query || {};

    if (!$query->to_string) {
	$c->res->code(301);
	$c->redirect_to("/");
	return;
    }

    if ($c->param('fhfilter')) {
	my $filter = $c->param('fhfilter');

	# check content type
	my $types = { story => "/",
		      stories => "/",
		      journal => "/journals/",
		      journals => "/journals/",
		      submission => "/submissions/",
		      submissions => "/submissions/",
		      comment => "/comments",
		      comments => "/comments",
		      poll => "/polls/",
		      polls => "/polls/",
		    };
	if ($types->{$filter}) {
	    $c->res->code(301);
	    $c->redirect_to("$types->{$filter}");
	    return;
	}

	 # check tag
	my $tag = $c->model('tags')->select_tagnames(tagname => $filter);
	if ($tag) {
	    $c->res->code(301);
	    $c->redirect_to("/tag/$filter");
	    return;
	}
    }

    my $section = $c->stash("section") || $c->param('section');
    if ($section) {
	if ($section eq 'mac') {
	    $c->res->code(301);
	    $c->redirect_to("/tag/apple");
	    return;
	}
	if (any { $section eq $_ } @sections) {
	    $c->res->code(301);
	    $c->redirect_to("/tag/$section");
	    return;
	}
	$c->render('Not found', status => '404');
	return;
    }

    if ($c->param('issue')) {
	if ($c->param('issue') =~ m/^(\d{2})(\d{2})(\d{2})(\d{2})$/) {
	    $c->res->code(301);
	    $c->redirect_to("/story/$2/$3/$4");
	    return;
	}
	$c->render('Not found', status => '404');
	return;
    }
}

sub firehose {
    my $c = shift;

    my $url = $c->req->url;
    my $query = $url->query || {};
    if (!$query->to_string) {
	$c->res->code(301);
	$c->redirect_to("/");
	return;
    }

    # check parameters
    if ($c->param('id')) {
	$c->_redirect_by_id;
	return;
    }

    if ($c->param('view')) {
	$c->_redirect_by_view;
	return;
    }

    # check other params
    $c->index;
}


1;

