package Newslash::Web::Controller::User;
use Mojo::Base 'Mojolicious::Controller';
use Data::Dumper;
use List::Util qw(any);

sub _base_render {
    my ($c, $tab, $f) = @_;
    my $users = $c->model('users');
    my $user = $c->stash('user');
    my $the_user = $user;
    my $nickname = $c->stash('nickname') || "";
    $nickname =~ s/\A~(.*)\z/$1/;

    if ($nickname) {
        $the_user = $users->select(nickname => $nickname);
        if (!$the_user) {
            $c->render('Not found', status => '404');
            return;
        }
    }
    $the_user->{config} = $users->configs->select(uid => $the_user->{uid});
    my $items = $f->($c, $the_user);

    my $relation = 0;
    if ($user->{uid} != $the_user->{uid}) {
        my $friends = $user->{relations}->{friends};
        my $foes = $user->{relations}->{foes};
        if (any { $the_user->{uid} eq $_ } keys %$friends) {
            $relation = 1;
        }
        if (any { $the_user->{uid} eq $_ } keys @$foes) {
            $relation = 2;
        }
    }

    my $page = { type => "user" };
    my $content_type_table = { home => "recent",
                               comments => "comment",
                               journals => "journal",
                               submissions => "submission"
                             };
    $page->{content_type} = $content_type_table->{$tab} || "user";

    $c->render(template => "user/base",
               page => $page,
               user_tab => $tab,
               items => $items,
               the_user => $the_user,
               relation => $relation
              );
    $c->stats->add_event_counter("userhome_view");

}

sub home {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('timeline')->select(uid => $the_user->{uid}, limit => 10);
        return $items;
    };
    $c->_base_render(home => $query_func);
}

sub journals {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('journals')->select(uid => $the_user->{uid},
                                                  order_by => { create_time => "DESC" },
                                                  limit => 10);
        return $items;
    };
    $c->_base_render(journals => $query_func);
}

sub comments {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('comments')->select(uid => $the_user->{uid},
                                                  order_by => { create_time => "DESC" },
                                                  limit => 10);
        return $items;
    };
    $c->_base_render(comments => $query_func);
}

sub submissions {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('submissions')->select(uid => $the_user->{uid},
                                                     order_by => { create_time => "DESC" },
                                                     limit => 10);
        return $items;
    };
    $c->_base_render(submissions => $query_func);
}

sub friends {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('users')->friends->friends(uid => $the_user->{uid});
        return $items;
    };
    $c->_base_render(friends => $query_func);
}

sub foes {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('users')->friends->foes(uid => $the_user->{uid});
        return $items;
    };
    $c->_base_render(foes => $query_func);
}

sub fans {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('users')->friends->fans(uid => $the_user->{uid});
        return $items;
    };
    $c->_base_render(fans => $query_func);
}

sub freaks {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('users')->friends->freaks(uid => $the_user->{uid});
        return $items;
    };
    $c->_base_render(freaks => $query_func);
}

sub achievements {
    my $c = shift;
    my $query_func = sub {
        my ($c, $the_user) = @_;
        my $items = $c->model('users')->achievements->select(uid => $the_user->{uid});
        return $items;
    };
    $c->_base_render(achievements => $query_func);
}


1;

