package Template::Plugin::DateTimeFormatter;
use strict;
use warnings FATAL => 'all';
our $VERSION = '1.00';

use Template::Plugin;
use Template::Iterator;
use Template::Exception;
use base qw(Template::Plugin);

use DateTime;
use DateTime::Format::MySQL;

use Time::Piece;
use Encode;
use Data::Dumper;

use constant MYSQL_FORMAT => '%Y-%m-%d %H:%M:%S';

sub new {
    my ($class, $context, @args) = @_;
    return bless { _CONTEXT => $context,
                 }, $class;
}

sub mysql_to_user {
    my ($self, $mysql_string, $offset_sec, $format) = @_;
    return if !$mysql_string;
    $offset_sec ||= 0;
    $format ||= '%Y/%m/%d %H:%M:%S';

    # parse mysql datetime
    my $dt = eval {Time::Piece->strptime($mysql_string, MYSQL_FORMAT);};
    if ($@) {
        warn "$@";
        return;
    }
    return if !$dt;

    $dt = gmtime($dt->epoch + $offset_sec);
    return decode_utf8($dt->strftime($format));
}

sub _mysql_to_user {
    my ($self, $mysql_string, $offset_sec, $format) = @_;
    return if !$mysql_string;

    my $dt =  eval {DateTime::Format::MySQL->parse_datetime($mysql_string);};
    return if ($@);
    return if !$dt;
    $dt->add(seconds => $offset_sec || 0);

    return $self->_format_datetime($dt, $format);

}

sub _format_datetime {
    my ($self, $dt, $format) = @_;
    $format ||= '%Y/%m/%d %H:%M:%S';
    return $dt->strftime($format);
}


1;
