/* article-item2.js */

var articleItem = {};

articleItem.init = function init () {
  function data() {
    var data = {
      // only editor's property will be send when preview/post 
      editor: {
        title: "",
        intro_text: "",
        body_text: "",
        create_time: "",
        author: "",
        dept: "",
        comment_status: "enabled",
        submissioncopy: 0,
        url: "",
        email: "",
        tags_string: "",
        related_urls: "",
        display: "1",
        posttype: 1,
        formatter: "modern",
        id: "",
      },

      message: "",
      createdUrl: "",
      currentTopics: [],
      
      enableAutoPreview: 1,
      mode: "",

      primaryTopicIconURL: "",
      cancelable: 0,
    };
    return data;
  }

  var props = {
    item: {
      type: Object,
      default: function () {return new Content();},
    },
    showEditor: {
      type: [Boolean, Number, String],
      default: false,
    },
    csrfToken: String,
  };

  /*
   * watch
   */
  var watch = {
    'editor.intro_text': watchIntrotext,
    'editor.body_text': watchBodytext,
    'editor.title': watchTitle,
    'editor.url': watchURL,
    'editor.create_time': watchCreatetime,
    'editor.author': watchAuthor,
    'editor.dept': watchDept,
    'editor.tags_string': updateTopics,
  };

  function watchIntrotext(val, oldVal) {
    this.item.intro_text = (val.length > 0) ? quoteHtml(val) : "";
  };

  function watchBodytext(val, oldVal) {
    this.item.body_text = (val.length > 0) ? quoteHtml(val) : "";
  };

  function watchTitle(val, oldVal) {
    this.item.title = (val.length > 0) ? quoteTitle(val) : "";
  };

  function watchURL(val, oldVal) {
    this.item.url = (val.length > 0) ? val : "";
  };

  function watchCreatetime(val, oldVal) {
    this.item.time_string = nsUtil.formatToLocalDateTime(new Date(val));
  };

  function watchAuthor(val, oldVal) {
    this.item.author = val;
  };

  function watchDept(val, oldVal) {
    this.item.dept = val;
  };

  /*
   * computed
   */
  var computed = {
    editable: editable,
    url: url,
  };

  function editable() {
    return (user.uid == this.item.uid);
  }

  function url() {
    if (this.item.content_type == 'story') {
      return "/story/" + this.item.sid + "/";
    } else if (this.item.content_type == 'journal') {
      return "/journal/" + this.item.id + "/";
    } else if (this.item.content_type == 'submission') {
      return "/submission/" + this.item.subid + "/";
    }
      return "";
  }

  /*
   * methods and relative private functions
   */
  var methods = {
    showPreview: showPreview,
    postItem: postItem,
    leavePreview: leavePreview,
    startEdit: startEdit,
    cancelEdit: cancelEdit,
    updateTopics: updateTopics,
    topicIconURL: topicIconURL,
    updatePrimaryTopicIconURL: updatePrimaryTopicIconURL,
  };

  function updatePrimaryTopicIconURL() {
    if (this.item.primary_topic) {
      this.primaryTopicIconURL = nsUtil.topicIconURL(this.item.primary_topic.keyword);
      return;
    }
    this.primaryTopicIconURL = "";
  }

  function showPreview(event) {
    var type = this.item.content_type;
    if (!type) {
      console.warn("no content_type!");
      return;
    }
    var postData = _preparePostData(this, "preview");

    if (type == "journal") {
      newslash.previewJournal(postData).then(
        (resp) => { // success
          this.message = "";
          this.previewTitle = resp.item.title;
          this.previewIntro = resp.item.intro_text;
          this.item.url = this.editor.url;
          this.item.primary_topic = resp.item.topic;
          this.updatePrimaryTopicIconURL();

          this.mode = "preview";
        },
        (resp) => { // fail
          if (resp.message) {
            this.message = resp.message;
          }
        }
      );
      return;
    }

    this.$newslash.post(type,
                        postData,
                        { csrfToken: this.csrfToken,
                          noCaptcha: 1 },
      (response) => { // success
        this.message = "";
        this.previewTitle = response.body.item.title;
        this.previewIntro = response.body.item.intro_text;
        this.item.primary_topic = response.body.item.topic;
        this.item.url = response.body.item.url;
        this.item.dept = response.body.item.dept;
        this.updatePrimaryTopicIconURL();

        this.mode = "preview";
      },
      (response) => { // fail
        if (response.body.message) {
          this.message = response.body.message;
        }
      }
    );
  }

  function postItem(event) {
    this.mode = "posting";
    var postData = _preparePostData(this, "post");
    var type = this.item.content_type;
    if (!type) {
      console.warn("no content_type!");
      return;
    }

    // journal post
    if (type == "journal") {
      postData.csrf_token = this.csrfToken;
      newslash.postJournal(postData).then(
        (resp) => { // success
          this.message = "";
          var id = resp.id;
          var type = resp.type;
          var url = '/' + type + '/' + id;
          
          // check if new post
          if (!postData.id) {
            this.createdUrl = url;
            this.item.subid = id;
          }
          this.message = "";
          this.mode = "posted";
        },
        (resp) => { // fail
          this.message = resp.message;
          this.mode = "preview";
        }
      );
      return;
    }

    // submission post
    this.$newslash.post(type, postData, {csrfToken: this.csrfToken},
                        (response) => { // success
                          this.message = "";
                          var id = response.body.url_id || response.body.id;
                          var type = response.body.type;
                          var url = '/' + type + '/' + id;
                          
                          // check if new post
                          if (!postData.id) {
                            this.createdUrl = url;
                            if (page.type == "submission") {
                              this.item.subid = id;
                            }
                            this.editor.id = response.body.id;
                          }
                          this.message = "";
                          this.mode = "posted";
                        },
                        (response) => { // fail
                          this.message = response.body.message;
                          this.mode = "preview";
                        }
                       );
    
  }

  function _preparePostData(vm, action) {
    action = action || "preview";

    vm.message = "";
    var postData = {};
    postData.item = {};
    postData.action = action;
    
    if (articleItem.recaptchaToken) {
      postData.recaptcha_token = articleItem.recaptchaToken;
    }

    Object.keys(vm.editor).forEach(k => {
      postData.item[k] = vm.editor[k];
    });

    // prepare ID
    if (vm.editor.id) {
      if (page.type == "story") {
        postData.item.stoid = vm.editor.id;
      }
    }

    return postData;
  }

  function leavePreview(event) {
    this.mode = "editing";
  }

  function quoteHtml(html) {
    return nsUtil.quoteHtml(html, siteConfig.editorConfig.allowed_tags);
  }

  function quoteTitle(html) {
    return nsUtil.quoteTitle(html, {});
  }

  function startEdit() {
    _startEdit(this);
  }

  function cancelEdit() {
    for (k in this.originalItem) {
      this.item[k] = this.originalItem[k];
    }
    this.mode = "";
  }

  function updateTopics() {
    var tags = this.editor.tags_string.split(" ");
    this.currentTopics.splice(0);
    if (tags.length == 0) {
      return;
    }
    var pushed = {};
    tags.forEach((tag) => {
      if (siteConfig.topics[tag]) {
        var keyword = siteConfig.topics[tag].keyword;
        if (!pushed[keyword]) {
          this.currentTopics.push(keyword);
          pushed[keyword] = 1;
        }
      }
    });
  }

  function topicIconURL(topic) {
    return nsUtil.topicIconURL(topic);
  }

  /*
   * created and related private functions
   */
  function created() {
    if (this.showEditor) {
      _startEdit(this);
    }
    this.updatePrimaryTopicIconURL();
  }

  function _startEdit(c) {
    _initEditor(c);
  }
  
  function _initEditor(c) {
    // backup original item
    // copy item property to c (component)
    c.originalItem = {};
    for (k in c.$data.item) {
      if (c.item[k]) {
        c.originalItem[k] = c.item[k];
      }
    }

    // set createtime
    if (c.item.createtime) {
      var dt = nsUtil.decodeMySQLDateTime(c.item.createtime);
      c.editor.createtime = nsUtil.formatToLocalISOString(dt);
    } else {
      c.editor.createtime = nsUtil.formatToLocalISOString(new Date());
    }
    c.editor.author = user.nickname;

    if (c.item.tags) {
      var tagnames = c.item.tags.map(x => {return x.tagname});
      c.editor.tags_string = tagnames.join(" ");
    }
    updateTopics.call(c);

    // journal edit mode
    if (page.content_type == 'journal') {
      c.item.content_type = "story";
      if (page.id) {
        // edit mode
        newslash.getJournal(page.id).then(
          resp => { // succeed
            var item = resp.journal;
            c.cancelable = 1;
            c.editor.title = item.title;
            c.editor.intro_text = item.article;
            c.editor.tags_string = item.tags_string || "";
            c.editor.comment_status = item.comment_status;
            c.editor.formatter = item.formatter;
            c.editor.posttype = item.posttype;
            c.editor.id = item.id;
          },
          resp => { // failed
            c.message = resp.body.message;
          });
      } else {
        // create
      }
    }

    // story edit mode
    if (page.content_type == 'story' && page.stoid) {
      c.cancelable = 1;
      c.item.content_type = "story";

      c.$newslash.getStoryByID(page.stoid,
        (resp) => {
          // succeed
          var item = resp.body.item;
          for (var k in c.editor) {
            if (item[k] !== undefined) {
              c.editor[k] = item[k];
            }
          }
          // convert tags to tags_string
          if (item.tags) {
            var tagnames = item.tags.map(tag => tag.tagname);
            c.editor.tags_string = tagnames.join(" ");
          }
          // TODO: related stories
          // convert public to display
          c.editor.display = (item.public == "yes");
          c.editor.createtime = nsUtil.formatToLocalISOString(nsUtil.decodeMySQLDateTime(item.create_time));
          c.editor.id = item.stoid;
        },
        (resp) => { //failed
          c.message = resp.body.message;
        });
    }
      
    // submission to story mode
    if (c.item.submission_id) {
      c.$newslash.getSubmissionByID(c.item.submission_id, 
        (resp) => { // succeed
          c.editor.title = resp.body.item.title;
          c.editor.intro_text = resp.body.item.drafttext;
          c.item.content_type = "story";
        },
        (resp) => { //failed
          c.message = resp.body.message;
        });
    }

    // check formatter
    if (c.editor.formatter == "legacy") {
      c.enableAutoPreview = 0;
    }
    c.mode = "editing";
  }

  /*
   * register component
   */
  var c = Vue.component('article-item', {
    template: '#article-item-template',
    props: props,
    data: data,
    computed: computed,
    watch: watch,
    methods: methods,
    created: created,
  });

};

/*
 * utilities
 */

articleItem.run = function (params) {
  params = params || {};
  if (!params.el) {
    console.error('error in articleItem.run(): no element given');
    return;
  }
  this.vm = new Vue({el: params.el});
};

articleItem.addTrigger = function addTrigger (selector, params) {
  params = params || {};
  if (!params.el) {
    console.error('error in articleItem.addTrigger(): no element given');
    return;
  }
  if (!selector) {
    console.error('error in articleItem.addTrigger(): no selector given');
    return;
  }
  $(selector).on('click', function (e) {
    $(params.el).children("article-item").attr("show-editor", 1); 
    articleItem.run(params);
  });
};

// articleItem.init();

