/* comment-tree.js */
var commentTree = {};

commentTree.run = function (params) {
  /* define exotic parameters */
  params = params || {};
  var userConfig = params.userConfig || {};
  var siteConfig = params.siteConfig || {};
  var pageInfo = params.pageInfo || {};
  var user = params.user || {};

  if (!params.el) {
    console.log('error in commentTree.run(): no element given');
    return;
  }

  /*
   * register <comment-tree>
   */
  Vue.component('comment-tree', {
    template: '#comment-tree-template',
    props: [
    ],
    data: function () {
      return {
        comments: [],
        commentAllowed: pageInfo.comment_allowed,
        archived: pageInfo.archived,
        discussionId: pageInfo.discussion_id,
        rootId: pageInfo.id,
        parentId: pageInfo.parent_id || 0,
      };
    },
    created: function commentTreeCreated() {
      newslash.getComments(this.discussionId, this.parentId).then(
        (resp) => { // success
          var comments = resp.comments;
          
          // build comment tree
          var commentIndex = {};
          comments.forEach(comment => {
            comment._children = [];
            commentIndex[comment.cid] = comment;
            if (comment.pid && commentIndex[comment.pid]) {
              commentIndex[comment.pid]._children.push(comment);
            }
          });
          this.comments = resp.comments;
        },
        (resp) => { // fail
          statusIndicator.error("comment_loading_error");
        }
      );
    },
  });

  /*
   * register <comment-header>
   */
  Vue.component('comment-header', {
    template: '#comment-header-template',
    data: function () {
      return {
        config: userConfig || {},
        savable: user.is_login || 0,
        commentAllowed: pageInfo.comment_allowed,
        archived: pageInfo.archived,
        showPrefs: false,
      };
    },
    methods: {
      saveConfig: function saveConfig() {
        const data = {
          type: "config",
          config: this.$data.config,
        };
        statusIndicator.loading("saving");
        this.$newslash.post('user', data,
                            (response) => { // success
                              statusIndicator.done("saved");
                            },
                            (response) => { // fail
                              statusIndicator.error("save_failed");
                            }
                           );
      }
    },
  });
  
  /*
   * register <comment>
   */
  Vue.component('comment', {
    template: '#comment-template',
    data: function () {
      return {
        modReasons: siteConfig.modReasons,
        moderateReason: 0,
        moderateMessage: '',
        metamoderateMessage: '',
        displayModInfo: 0,
        loadingMods: 1,
        noMods: 0,
        secToken: '',
        showShareButtons: false,

        displayForce: false,
        commentAllowed: pageInfo.comment_allowed,
        archived: pageInfo.archived || false,

        isFormVisible: false,
      };
    },
    props: {
      comment: Object,
      preview: { type: Boolean, default: false },
    },
    created: function () {
      if (this.comment._displayForce) {
        this.displayForce = true;
      }
    },
    computed: {
      commentLocalTime: function () {
        if (this.comment.create_time == "") return "";
        var dt = newslash.util.decodeMySQLDateTime(this.comment.create_time);
        return newslash.util.formatToLocalDateTime(dt);
      },
      isIpidVisible: function () { return user.is_admin; },
      isM2able: function () { return user.is_login; },
      signedPoints: function () {
        var num = this.comment.points;
        if (num < 0) {
          return "-" + num;
        } else if (num > 0) {
          return "+" + num;
        }
        return "0";
      },
      reasonText: function () {
        if (this.comment.reason != 0) {
          return "：" + siteConfig.modReasons[this.comment.reason];
        }
        return "";
      },
      isScoreVisible: function () {
        return !userConfig.comment.hide_score
          && this.comment.lastmod != 0
          && this.comment.points != 0;
      },
      isFoldedComment: function () {
        return this.comment.points < userConfig.comment.fold_threshold
          && !this.comment.isPreview
          && !this.displayForce;
      },
      isHiddenComment: function () {
        return this.comment.points < userConfig.comment.show_threshold
          && !this.comment.isPreview
          && !this.displayForce;
      },
      isHiddenBoxMode: function () {
        if (this.comment.isPreview
            || this.displayForce
            || this.comment.points >= userConfig.comment.show_threshold) {
          return 0;
        }
        // 子アイテムの少なくとも1つが表示なら -> 「1件の非表示コメント」表示
        var count = _countHiddenItems(this.comment);
        if (count[0] != count[1]) {
          return 1;
        }
        return 0;
      },
      hiddenCount: function () {
        // when point >= threshold: hide "hidden" box.
        if (this.comment.points >= userConfig.comment.show_threshold) {
          return 0;
        }
        // 子アイテムの少なくとも1つが表示なら -> 「1件の非表示コメント」表示
        const count = _countHiddenItems(this.comment);
        if (count[0] != count[1]) {
          return 1;
        }
        // 子アイテムがすべて非表示なら -> 「n件の非表示コメント」表示
        return count[0];
      },
      isModerateEnabled: function () {
        return this.comment.cid && (user.is_admin || (user.is_moderator && this.commentAllowed));
      },
      isReplyEnabled: function () {
        return this.commentAllowed && !this.comment.isPreview;
      },
      isSignatureVisible: function () {
        return !userConfig.comment.hide_signature && this.comment.signature;
      },
    },
    methods: {
      //isM2able: isM2able, 
      showItem: showItem,
      showModInfo: showModInfo,
      moderate: moderate,
      metamoderate: metamoderate, 
      showReplyForm: showReplyForm,
      hideReplyForm: hideReplyForm,
      toggleShareButtons: toggleShareButtons,
    },
  });

  /* methods for comment */
  /*
  function isM2able(mod) {
    return typeof mod.m2val == 'undefined' ? 1 : 0;
  }
   */

  function showModInfo() {
    this.loadingMods = 1;
    this.displayModInfo = !this.displayModInfo;
    if (!this.displayModInfo) {
      return;
    }
    // get moderation data
    //this.$http.get(this.urls.moderate + "?cid=" + this.comment.cid).then(
    this.$newslash.getModerations(this.comment.cid,
      (response) => { // success
        if (response.body.sec_token) {
          this.secToken = response.body.sec_token;
        }
        for (var i = 0; i < response.body.moderations.length; i++) {
          response.body.moderations[i].status = '';
        }
        if (response.body.moderations.length > 0) {
          Vue.set(this.comment, 'moderations', response.body.moderations);
        } else {
          Vue.set(this.comment, 'moderations', []);
          this.noMods = 1;
        }
        this.loadingMods = 0;
      },
      (response) => { // fail
      }
    );
  }

  function showReplyForm() {
    this.isFormVisible = true;
  }

  function hideReplyForm() {
    this.isFormVisible = false;
  }

  function toggleShareButtons() {
    this.showShareButtons = !this.showShareButtons;
  }

  function showItem() {
    this.displayForce = true;
  }

  function moderate() {
    var data = {
      reason: Number(this.moderateReason),
      discussion_id: pageInfo.discussion_id,
      cid: Number(this.comment.cid),
      action: 'moderate',
    };
    this.loadingMods = 1;
    this.$http.post(this.urls.moderate, data).then(
      (response) => { // success
        this.moderateMessage = "保存しました";
      },
      (response) => { // fail
        if (response.body.message) {
          this.moderateMessage = "エラー：" + response.body.message;
        } else {
          this.moderateMessage = "エラー";
        }
      }
    );
  }

  function metamoderate(mod, value) {
    if (mod.m2val == 1 || mod.m2val == -1) {
      return;
    }
    var data = {
      value: value,
      moderation_id: mod.id,
      action: 'metamoderate',
      sec_token: this.secToken,
    };

    mod.status = 'm2saving';
    this.$http.post(this.urls.metamoderate, data).then(
      (response) => { // success
        mod.status = 'm2saved';
        mod.m2val = value;
      },
      (response) => { // fail
        mod.status = 'error';
        if (response.body.message) {
          console.log(response.body.message);
        }
      }
    );
  }
  
  function _countHiddenItems(comment) {
    if (!comment || !comment._children) {
      console.log("_countHiddenItems: invalid parameter given");
      return [0, 0];
    }
    var result = [1, 0];
    for (var i = 0; i < comment._children.length; i++) {
      var ret = _countHiddenItems(comment._children[i]);
      result[0] += ret[0];
      result[1] += ret[1];
    }
    if (comment.points < userConfig.comment.show_threshold) {
      result[1]++;
    }
    return result;
  }

  /*
   * register <comment-form>
   */
  Vue.component('comment-form', {
    template: '#comment-form-template',
    data: function () {
      return {
        isFormVisible: true,
        isPreviewVisible: false,
        cancelable: true,
        rawTitle: "",
        rawComment: "",
        title: "",
        comment: "",
        author: "",
        date: "",
        signature: "",
        preview: new Comment(),
        csrfToken: "",
      };
    },
    props: {
      replyTo: Object,
      target: Array,
    },
    created: function () {
      if (this.replyTo) {
        this.rawTitle = "Re: " + this.replyTo.title;
      } else {
        this.cancelable = false;
      }
    },
    computed: {
      isItemFilled: function isItemFilled () { return this.rawTitle.length && this.rawComment.length;}
    },
    methods: {
      cancelEdit: cancelEdit,
      doPreview: doPreview,
      leavePreview: leavePreview,
      postComment: postComment,
    },
    
  });

  function cancelEdit() {
    this.$emit("finish-edit");
  }

  function doPreview() {
    var data = {
      title: this.rawTitle,
      comment: this.rawComment,
      discussion_id: pageInfo.discussion_id,
      stoid: pageInfo.stoid,
      pid: this.replyTo ? this.replyTo.pid : 0,
      action: 'preview',
    };
    this.$newslash.post("comment", data, {noCaptcha: 1},
                        (response) => { // success
                          this.preview.title = response.body.title;
                          this.preview.comment = response.body.comment;
                          this.preview.author = response.body.author;
                          this.preview.date = response.body.date;
                          this.preview.signature = response.body.signature;
                          this.csrfToken = response.body.csrf_token || "";
                          this.preview.isPreview = true;

                          this.isFormVisible = false;
                          this.isPreviewVisible = true;

                        },
                        (response) => { // fail
                          if (response.body.message) {
                            statusIndicator.error(response.body.message);
                          }
                        }
                       );
  }

  function leavePreview() {
    this.isFormVisible = true;
    this.isPreviewVisible = false;
  }

  function postComment() {
    var data = {
      title: this.rawTitle,
      comment: this.rawComment,
      discussion_id: pageInfo.discussion_id,
      stoid: pageInfo.stoid,
      pid: 0,
      action: 'post',
      csrf_token: this.csrfToken,
    };

    if (this.replyTo) {
      data.pid = this.replyTo.cid;
    } else if (pageInfo.comment_id) {
      data.pid = pageInfo.comment_id;
    }

    this.$newslash.post("comment", data,
      (response) => { // success
        var newComment = response.body.new_comment;
        newComment._children = [];
        newComment._displayForce = true;
        //comments.initComment(newComment, this.reply.parent);
        
        this.isFormVisible = true;
        this.isPreviewVisible = false;

        if (this.replyTo) {
          this.replyTo._children.unshift(newComment);
        } else {
          this.target.push(newComment);
        }
        this.rawTitle = this.replyTo ? ("Re: " + this.replyTo.title) : "";
        this.rawComment = "";
        this.$emit("finish-edit");
      },
      (response) => { // fail
        if (response.body.message) {
          statusIndicator.error(response.body.message);
        }
      }
    );
  }

  this.vm = new Vue({el: params.el});
};

