/* user-settings.js */
var userSettings = {};

userSettings.run = function run (params) {
  Vue.component('message', {
    props: ["target", ],
    template: '#settings-message',
  });

  var vm;
  var userConfig = params.userConfig || userConfig || {};
  var user = params.user || user || {};

  var data = {
    user: user,

    infoSettings: userConfig.info || {},
    infoSettingsMessage: "",

    emailSettings: { email: params.email,
                     newEmail: "" },
    emailSettingsMessage: "BLANK_ADDRESS",
    
    commentSettings: userConfig.comment || {},
    commentSettingsMessage: "",
    
    messageSettings: [],
    messageSettingsMessage: "",

    passwordSettings: { current: "",
                        new1: "",
                        new2: "" },
    passwordSettingsMessage: "BLANK_PASSWORD",
    state: "",
  };

  function watchEmail(val, oldVal) {
    if (val.newEmail.length == 0) {
      this.emailSettingsMessage = "BLANK_ADDRESS";
    } else {
      this.emailSettingsMessage = "";
    }
  }

  function watchPasswords(val, oldVal) {
    if (val.current.length == 0 || val.new1.length == 0) {
      this.passwordSettingsMessage = "BLANK_PASSWORD";
    } else {
      this.passwordSettingsMessage = "";
    }

    if (val.new1 != val.new2) {
      this.passwordSettingsMessage = "PASSWORD_NOT_MATCH";
    }
  }

  var methods = {
    saveInfoSettings: function saveInfoSettings() {
      var data = { info: this.infoSettings };
      this.state = "posting";
      newslash.updateSettings(data).then(
        (resp) => {
          this.infoSettingsMessage = "SAVED";
          this.state = "";
          // succeed
        },
        (resp) => {
          // failed
          this.infoSettingsMessage = "SERVER_ERROR";
          this.state = "";
        });
    },
    requestEmailChange: function requestEmailChange() {
      this.state = "posting";
      newslash.requestEmailChange(this.emailSettings.newEmail).then(
        (resp) => {
          this.emailSettingsMessage = "REQUEST_OK";
          this.state = "";
          // succeed
        },
        (resp) => {
          // failed
          this.emailSettingsMessage = resp.message;
          this.state = "";
        });
    },
    saveCommentSettings: function saveCommentSettings() {
      var data = { comment: this.commentSettings };
      this.state = "posting";
      newslash.updateSettings(data).then(
        (resp) => {
          this.commentSettingsMessage = "SAVED";
          this.state = "";
          // succeed
        },
        (resp) => {
          // failed
          this.commentSettingsMessage = "SERVER_ERROR";
          this.state = "";
        });
    },
    saveMessageSettings: function saveMessageSettings() {
      this.state = "posting";
      newslash.updateMessageSettings(this.messageSettings).then(
        (resp) => {
          this.messageSettingsMessage = "SAVED";
          this.state = "";
          // succeed
        },
        (resp) => {
          // failed
          this.messageSettingsMessage = "SERVER_ERROR";
          this.state = "";
        });
    },
    savePasswordSettings: function savePasswordSettings() {
      this.state = "posting";
      newslash.updatePassword(this.passwordSettings.current, this.passwordSettings.new1).then(
        (resp) => {
          this.passwordSettingsMessage = "SAVED";
          this.state = "";
          // succeed
        },
        (resp) => {
          // failed
          this.passwordSettingsMessage = resp.message;
          this.state = "";
        });
    },
    setMessageMode: function setMessageMode(code, mode) {
      vm.messageSettings.forEach(conf => {
        if (conf.code == code) {
          conf.mode = mode;
          this.messagesSaved = 0;
        }
      });
    },
  };

  vm = new Vue({el: params.el,
                data: data,
                methods: methods,
                created: function () {
                  newslash.getMessageSettings().then(
                    (resp) => { // success
                      resp.config.forEach(conf => {
                        vm.messageSettings.push(conf);
                      });
                    },
                    (resp) => { // fail
                    }
                  );
                },
               });
  vm.$watch('passwordSettings', watchPasswords, { deep: true });
  vm.$watch('emailSettings', watchEmail, { deep: true });
};
