/* user-settings.js */
const userSidebar = {};
var vm;

userSidebar.run = function run (params) {
  var user = params.user || {};
  var data = {
    userConfig: params.userConfig || {},
    user: user || {},
    user_id: params.user_id,
    message: "",
    urls: {
      get: '/api/v1/sidebar/item',
      save: '/api/v1/user',
      user: '/api/v1/user?type=sidebar',
    },
    boxes: [],
    userBoxes: [],
    selectedBoxes: [],
    selectedUserBoxes: [],
  };

  if (params.user_id) {
    data.urls.user = data.urls.user + "&user_id=" + params.user_id;
  }

  var computed = {
    isBoxMovable: function () {
      return (this.selectedUserBoxes.length == 1);
    },
    isBoxDeletable: function () {
      return (this.selectedUserBoxes.length > 0);
    },
    isBoxAddable: function () {
      return (this.selectedBoxes.length > 0);
    },
  };

  var methods = {
    addBox: function () {
      for (var i = 0; i < this.boxes.length; i++) {
        if (this.selectedBoxes.find(e => { return e == this.boxes[i].id; })) {
          this.userBoxes.push(this.boxes[i]);
          this.boxes[i].used = 1;
        }
      }
    },
    deleteBox: function () {
      for (var i = 0; i < this.userBoxes.length; i++) {
        if (this.selectedUserBoxes.find(e => { return e == this.userBoxes[i].id; })) {
          this.userBoxes[i].used = 0;
          this.userBoxes.splice(i, 1);
        }
      }
    },
    moveBoxUp: function () {
      var targetID = this.selectedUserBoxes[0];
      for (var i = 0; i < this.userBoxes.length; i++) {
        if (this.userBoxes[i].id == targetID) {
          if (i == 0) {
            return;
          }
          var target = this.userBoxes[i];
          this.userBoxes.splice(i, 1);
          this.userBoxes.splice(i-1, 0, target);
          return;
        }
      }
    },
    moveBoxDown: function () {
      var targetID = this.selectedUserBoxes[0];
      for (var i = 0; i < this.userBoxes.length - 1; i++) {
        if (this.userBoxes[i].id == targetID) {
          var target = this.userBoxes[i];
          this.userBoxes.splice(i, 1);
          this.userBoxes.splice(i+1, 0, target);
          return;
        }
      }
    },
    saveConfig: function () {
      var boxes = [];
      for (var i = 0; i < this.userBoxes.length; i++) {
        boxes.push(this.userBoxes[i].id);
      }
      var data = {
        type: "sidebar",
        items: boxes,
        user_id: this.user_id,
      };
      this.$http.post(this.urls.save, data).then(
        (resp) => { // success
          this.message = "保存しました";
        },        
        (resp) => { // fail
          this.message = resp.body.message || "エラー";
        }
      );
    },
  };

  function getBoxes(vue) {
    // get sidebar boxes
    vue.$http.get(vue.urls.get).then(
      (resp) => { // success
        vue.boxes = resp.body.items || [];
        vue.message = resp.body.message || '';
        vue.boxes.forEach(box => {
          Vue.set(box, "used", 0);
        });
        getUserBoxes(vue);
      },
      (resp) => { // fail
        vue.boxes = [];
        vue.message = resp.body.message || "failed to get items";
      }
    );
  }

  function getUserBoxes(vue) {
    // get user sidebar settings
    vue.$http.get(vue.urls.user).then(
      (resp) => { // success
        resp.body.items.forEach(id => {
          for (var i = 0; i < vue.boxes.length; i++) {
            if (vue.boxes[i].id == id) {
              vue.userBoxes.push(vue.boxes[i]);
              vue.boxes[i].used = 1;
            }
          }
        });
        vue.message = resp.body.message || '';
      },
      (resp) => { // fail
        vue.message = resp.body.message || "failed to get user settings";
      }
    );
  }

  var created = function created() {
    getBoxes(this);
  };

  vm = new Vue({el: params.el,
                data: data,
                computed: computed,
                methods: methods,
                created: created,
               });
};
