# -*-Perl-*-
# test for feeds related api
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";

my $filters = $t->app->model('filters');
ok($filters, "get Filters instance");

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("apifiltertestadmin", $test_passwd);
    ok($admin, "create admin user");
}

subtest 'create/select/update/delete/exports filters and items' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $test_name = "test_filter";
    my $test_regexp = "foobarhogehoge";
    my $test_field = "/comment";
    my $test_pathes = "/comment";

    # prepare for test
    my $rs = $filters->select(name => $test_name);
    $filters->delete(filter_id => $rs->{filter_id}) if $rs;

    # login
    $test_man->login($admin);

    # create filter
    my $params = { action => "create",
                   name => $test_name,
                   description => "test description",
                   message => "error message",
                   regexp => $test_regexp,
                   fields => $test_field,
                   pathes => $test_pathes,
                 };
    $t->post_ok('/api/v1/admin/filter' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_has('/filter_id');
    my $filter_id = 0;
    if ($t->tx->res->json) {
        if ($t->tx->res->json->{message}) {
            diag $t->tx->res->json->{message};
        }
        $filter_id = $t->tx->res->json->{filter_id};
    }

    # list
    $t->get_ok('/api/v1/admin/filter')
      ->status_is(200)
      ->json_has('/items');

    # get
    $t->get_ok("/api/v1/admin/filter?filter_id=$filter_id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/filter_id' => $filter_id);

    # update
    $params = { action => "update",
                description => "新しい説明",
                filter_id => $filter_id
              };
    $t->post_ok('/api/v1/admin/filter' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_is('/filter_id', $filter_id);

    # check updated item
    $t->get_ok("/api/v1/admin/filter?filter_id=$filter_id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/description' => "新しい説明");

    # export
    unlink("/var/tmp/filters.json");
    $t->get_ok("/api/v1/admin/repository/export?target=filters")
      ->status_is(200);
    diag $t->tx->res->json->{message} if !$t->success;
    ok(-e "/var/tmp/filters.json", "filters.json generated");

    # import
    $t->get_ok("/api/v1/admin/repository/import?target=filters")
      ->status_is(200);
    diag $t->tx->res->json->{message} if !$t->success;
    unlink("/var/tmp/filters.json");

    # delete
    $params = { action => "delete",
                filter_id => $filter_id
              };
    $t->post_ok('/api/v1/admin/filter' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_is('/error', 0);

    # check deleted item
    $t->get_ok("/api/v1/admin/filter?filter_id=$filter_id")
      ->status_is(404);

    # finish
    $test_man->logout;
};

$test_man->cleanup;

done_testing();
