# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Mojo::Util qw(dumper);

use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);

my $users = $t->app->model('users');
ok($users, "get Users instance");
my ($user, $story);

if ($t->app->mode eq 'test') {
    $user = $test_man->create_user("tagtest01", "foobar");
    ok($user, "create test user");
    $story = $test_man->create_story(user => $user, title => 'This is test', body => '<p>This is test!</p>');
    ok($story, "create story for test");
}



SKIP: {
    skip "mode is not 'test'", 0 if ($t->app->mode ne 'test');

    my $tags = $t->app->model('tags');
    ok($tags, "get tags model");

    my $globjid = $story->{globjid};
    ok($globjid, "get story globjid");

    $test_man->login($user);

    my $test_data = { action => "set",
                      params => { name => "nod",
                                  globjid => $globjid, },
                    };

    # set
    $t->post_ok('/api/v1/tagmoderation' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/result/tag_id');
    diag $t->tx->res->json->{error}->{message} if $t->tx->res->code != 200;
    my $ts = $tags->select(globjid => $globjid,
                           uid => $user->{uid},
                           tagname => "nod");
    ok($ts, "select tag");
    ok(@$ts > 0, "retrive valid tag");

    # vote
    $test_data->{action} = "vote";
    $test_data->{params}->{name} = "nix";
    $t->post_ok('/api/v1/tagmoderation' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/result/tag_id');
    diag $t->tx->res->json->{error}->{message} if $t->tx->res->code != 200;

    # check if "nod" tag is deleted
    $ts = $tags->select(globjid => $globjid,
                           uid => $user->{uid},
                           tagname => "nod");
    ok($ts, "select tag");
    is(@$ts, 0, "retrive no tag");

    # check nix tag is set
    $ts = $tags->select(globjid => $globjid,
                           uid => $user->{uid},
                           tagname => "nix");
    ok($ts, "select tag");
    isnt(@$ts, 0, "retrive valid tag");

    # unset
    $test_data->{action} = "unset";
    $t->post_ok('/api/v1/tagmoderation' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_is('/result/delete_count', 1);
    diag $t->tx->res->json->{error}->{message} if $t->tx->res->code != 200;

    # check nix tag is deleted
    $ts = $tags->select(globjid => $globjid,
                           uid => $user->{uid},
                           tagname => "nix");
    ok($ts, "select tag");
    is(@$ts, 0, "retrive valid tag");

    $test_man->logout;
};

$test_man->cleanup;

done_testing();
