# -*-Perl-*-
# Model::Events tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

my $t = Test::Mojo->new('Newslash::Web');
my $events = $t->app->model('events');
ok($events, "get Events instance");

subtest 'create, select, delete' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    # delete db
    ok(defined $events->delete_all, "delete all events");

    # create event
    my $event_id1 = $events->create(event_type => "test",
                                   event_subtype => "test",
                                   related_id => 0,
                                   source_user_id => 0);
    ok($event_id1, "event 1 create");

    my $event_id2 = $events->create(event_type => "test",
                                   event_subtype => "test",
                                   related_id => 0,
                                   source_user_id => 0);
    ok($event_id2, "event 2 create");

    my $event = $events->select(event_id => $event_id1);
    ok($event, "event select");
    #diag(Dumper($event))

    my $oldest = $events->oldest;
    is_deeply($oldest, $event, "get oldest event");

    my $items = $events->fetch;
    ok($items, "fetch events");
    my $latest = $items->[0];
    ok($latest, "fetched event is valid");
    diag($events->last_error) if !$latest;
    is_deeply($latest, $event, "fetch oldest event");

    my $rs = $events->update(event_id => $event_id2, active => 0);
    ok($rs, "update event 01");
    $event = $events->select(event_id => $event_id2);
    is($event->{active}, 0, "event->{active} updated");

    $rs = $events->update(event_id => $event_id2, result => -1);
    ok($rs, "update event 02");
    $event = $events->select(event_id => $event_id2);
    is($event->{result}, -1, "event->{result} event updated");

    $rs = $events->delete(event_id => $event_id1);
    ok($rs, "event 1 delete");
    $rs = $events->delete(event_id => $event_id2);
    ok($rs, "event 2 delete");
};

done_testing();
