# -*-Perl-*-
# Model::Messages tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Storable;
use Newslash::Util::TestMan;


my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);

my $web_messages = $t->app->model('web_messages');
ok($web_messages, "get WebMessages instance");
my $messages = $t->app->model('messages');
ok($messages, "get Messages instance");
my $users = $t->app->model('users');
ok($users, "get Users instance");

# prepare for test
my ($user1, $user2);
if ($t->app->mode eq 'test') {
    $user1 = $test_man->create_user("msgtest1");
    ok($user1, "create test user1");
    $user2 = $test_man->create_user("msgtest2");
    ok($user2, "create test user2");
}

subtest 'basic message type methods' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $type = {
                message_type_id => 3333,
                name => "test_message_foo",
                description => "テスト用メッセージタイプ",
                seclev => 100,
                selectable_mode => 1,
               };
    my $created_id = $messages->create_message_type($type->{message_type_id},
                                                    $type->{name},
                                                    $type->{description},
                                                    $type->{seclev},
                                                    $type->{selectable_mode}
                                                   );
    is($created_id, $type->{message_type_id}, "create message type");

    my $new_obj = $messages->select_message_type(message_type_id => $type->{message_type_id});
    ok($new_obj, "select message type by type id");
    is_deeply($new_obj, $type, "select by message_type_id");

    $new_obj = $messages->select_message_type(name => $type->{name});
    ok($new_obj, "select message type by name");
    is_deeply($new_obj, $type, "select by name");

    my $rs = $messages->select_message_type;
    ok($rs, "select with no arguments");
    ok(@$rs > 0, "select some items with no arguments");

    $rs = $messages->delete_message_type(message_type_id => $type->{message_type_id});
    ok($rs, "delete message type");
};


subtest 'message create and select' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    # set reply message's notification to web
    my $rs = $users->messages->set(uid => $user2->{uid},
                                   code => 7,
                                   mode => 1);
    ok(defined $rs, "change notification mode");
    my $msg = $users->messages->select(uid => $user2->{uid},
                                       code => 7);
    ok($msg, "get updateded message");
    ok($msg->{mode} == 1, "message update succeeded");
    diag Dumper $msg if $msg->{mode} != 1;

    # update messages setting
    $rs = $users->messages->set(uid => $user2->{uid},
                                name => "newsletter",
                                mode => 0);
    ok(defined $rs, "change notification mode (2)");
    $msg = $users->messages->select(uid => $user2->{uid},
                                    name => "newsletter");
    ok($msg, "get message info by name");
    ok($msg->{mode} == 0, "message update succeeded (2)");
    diag Dumper $msg if $msg->{mode} != 0;

    my $title = "this is test";
    my $body = "this is test message.";
    my $msg_id = $messages->create(from => $user1->{uid},
                                   to => $user2->{uid},
                                   title => $title,
                                   body => $body,
                                   code => 7, # comment for journal,
                                   type => "notify");
    ok($msg_id, "create message");

    # get message
    $msg = $web_messages->select(id => $msg_id);
    ok($msg, "message created");
    is(ref($msg), "HASH", "web_messages->select returns hashref");
    if ($msg && ref($msg) eq "HASH") {
        is($msg->{user}, $user2->{uid}, "check message user");
        is($msg->{fuser}, 0, "check message fuser");
        is($msg->{code}, 7, "check message code");
        is($msg->{subject}, $title, "check message subject");
        is($msg->{message}, $body, "check message");
    }

    ok($web_messages->hard_delete(id => $msg_id), "delete message");
};

$test_man->cleanup;
done_testing();
