# -*-Perl-*-
# Newslash::Model::Moderations related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

use Newslash::Util::Test qw(create_user delete_user
                            create_story delete_story
                            create_comments
                            create_admin_user
                            create_users
                            get_anonymous_user
                          );

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');
my $moderations = $t->app->model('moderations');
my $users = $t->app->model('users');
my $comments = $t->app->model('comments');
my $stories = $t->app->model('stories');


my $test_users;
my $story;
my $admin;
my $anonymous;

# prepare for test
if ($t->app->mode eq 'test') {
    # at first, create test users.
    $test_users = create_users($t->app, "modtest", 4);
    $admin = create_admin_user($t->app, "mmtest_admin");
    ok($test_users, "create test users");
    ok($admin, "create admin user");

    # second, create story.
    $story = create_story($t->app, $admin, "メタモデテスト", "<p>メタモデテスト本文</p>");
    ok($story, "create story for test");

    $anonymous = get_anonymous_user($t->app);
    ok($anonymous, "get anonymous user");
}


SKIP: {
    skip "mode is not 'test'", 1 if ($t->app->mode ne 'test');
    my $discussion_id = $story->{discussion};

    my ($u_before, $u_after);
    my ($c_before, $c_after);

    # create comment 01
    my $comment_params;
    $comment_params = {
                       discussion_id => $discussion_id,
                       title => "コメントテスト01",
                       comment => "コメントテスト01の本文です。",
                      };
    my $cid01 = $comments->create($comment_params, $anonymous);
    ok($cid01, "comment creation 01");
    #diag("comment01: $cid01");

    # get comment 01
    my $comment01 = $comments->select(cid => $cid01);
    ok($comment01, "select comment 01");

    # get user modtest2 users_info
    $u_before = $users->info->select(uid => $test_users->{modtest2}->{uid});
    ok($u_before, "get userinfo for modtest2");

    # create comment 02
    $comment_params = {
                       discussion_id => $discussion_id,
                       title => "Re: コメントテスト01",
                       comment => "コメントテスト02の本文です。コメントテスト01の子コメントです。",
                       pid => $cid01,
                      };
    my $cid02 = $comments->create($comment_params, $test_users->{modtest2});
    ok($cid02, "comment creation 02");
    #diag("comment02: $cid02");

    # get user modtest2 users_info and check diff
    $u_after = $users->info->select(uid => $test_users->{modtest2}->{uid});
    is($u_after->{totalcomments}, $u_before->{totalcomments} + 1, "totalcomments changed");

    # get comment 02
    my $comment02 = $comments->select(cid => $cid02);
    ok($comment02, "select comment 02");

    # create comment 03
    $comment_params = {
                       discussion_id => $discussion_id,
                       title => "コメントテスト03",
                       comment => "コメントテスト03の本文です。",
                      };
    my $cid03 = $comments->create($comment_params, $test_users->{modtest3});
    ok($cid03, "comment creation 03");
    #diag("comment03: $cid03");

    # get comment03
    my $comment03 = $comments->select(cid => $cid03);
    ok($comment03, "select comment 03");

    # get moderation reasons
    my $reasons = $moderations->reasons;
    ok($reasons, "get reasons");

    # moderation comment01
    $u_before = $users->info->select(uid => $admin->{uid});
    $c_before = $comments->select(cid => $comment01->{cid});
    my $rs = $moderations->create($admin, $comment01->{cid}, 8); # id 8 is おもしろおかしい
    ok($rs, "plus moderation");
    diag($moderations->last_error) if !$rs;

    # check if totalmods is changed?
    $u_after = $users->info->select(uid => $admin->{uid});
    is($u_after->{totalmods}, $u_before->{totalmods} + 1, "totalmods changed");

    # check if points is updated?
    $c_after = $comments->select(cid => $comment01->{cid});
    isnt($c_after->{points}, $c_before->{points}, "points changed");

    # moderation comment02
    #diag("target: $comment02->{uid}");
    $c_before = $comments->select(cid => $comment02->{cid});
    $u_before = $users->info->select(uid => $comment02->{uid});
    $rs = $moderations->create($admin, $comment02->{cid}, 3); # id 3 is 荒し
    ok($rs, "minus moderation");
    diag($moderations->last_error) if !$rs;

    # check if karma is updated?
    $u_after = $users->info->select(uid => $comment02->{uid});
    isnt($u_after->{karma}, $u_before->{karma}, "karma changed");
    is($u_after->{downmods}, $u_before->{downmods} + 1, "downmods changed");

}

# cleanup
if ($t->app->mode eq 'test') {
    for my $u (keys %$test_users) {
        delete_user($t->app, $u);
    }
    delete_user($t->app, $admin->{nickname});
    delete_story($t->app, $story->{stoid});
}


done_testing();
