# -*-Perl-*-
# Model::Users tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

use Newslash::Util::Test qw(create_user delete_user
                            create_story delete_story
                            create_comments
                            create_admin_user
                            create_users
                            get_anonymous_user
                          );

my $t = Test::Mojo->new('Newslash::Web');
my $networks = $t->app->model('networks');
ok($networks, "get Networks instance");

my ($admin, $user, $anonymous, $story);

# prepare for test
if ($t->app->mode eq 'test') {
    #delete_user($t->app, 'nwtest');
    #delete_user($t->app, 'nwtest_admin');
    # at first, create test users.
    $user = create_user($t->app, "nwtest");
    $admin = create_admin_user($t->app, "nwtest_admin");
    ok($user, "create test user");
    ok($admin, "create admin user");

    # second, create story.
    $story = create_story($t->app, $admin, "テスト", "<p>テスト本文</p>");
    ok($story, "create story for test");

    # create comment
    my $comment_params;
    $comment_params = {
                       discussion_id => $story->{discussion},
                       title => "コメントテスト01",
                       comment => "コメントテスト01の本文です。",
                      };
    my $cid = $t->app->model('comments')->create($comment_params, $user);
    ok($cid, "comment creation");

    $anonymous = get_anonymous_user($t->app);
    ok($anonymous, "get anonymous user");
}

if ($t->app->mode eq 'test') {
    my $acts = $networks->activities->select(uid => $user->{uid});
    ok($acts, "get activities");
    ok(@$acts, "get valid activities");
    is($acts->[0]->{uid}, $user->{uid}, "valid contents (uid)");
    is($acts->[0]->{nickname}, "nwtest", "valid contents (nickname)");
    ok($acts->[0]->{count} > 0, "valid contents (count)");
    ok($acts->[0]->{first}, "valid contents (first)");
    ok($acts->[0]->{last}, "valid contents (last)");
}


# clean up
if ($t->app->mode eq 'test') {
    delete_user($t->app, 'nwtest');
    delete_user($t->app, 'nwtest_admin');
    delete_story($t->app, $story->{stoid});
}

done_testing();
