# -*-Perl-*-
# Newslash::Model::Submissions related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');
my $user;
my $test_man = Newslash::Util::TestMan->new($t);

my $submissions = $t->app->model('submissions');
ok($submissions, "get Submissions instance");

# prepare for test
if ($t->app->mode eq 'test') {
    # at first, create test users.
    $user = $test_man->create_user("storytest");
    ok($user, "create user");
}


subtest 'create/select/update submissions' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $params = {
                  user => $user,
                  title => "test story for submission",
                  url => "http://example.com/",
                  email => 'foo@example.com',
                  introtext => '<p>foo bar hoge</p>',
                  bodytext => '',
                  tags_string => "internet foo bar", # tid is 1913
                  #topics_chosen => {49 => 10, 2271 => 20},
                  tid => 1913,
                 };
    my $subid = $submissions->create(%$params);
    ok($subid, "create submission");
    diag $submissions->last_error if !$subid;

    my $subm = $submissions->select(submission_id => $subid);
    ok($subm, "select story by submission_id");

    my $dt = DateTime->now();
    my $c;
    my $year = $dt->year;
    my $month = $dt->month;
    my $day = $dt->day;
    my $offset = $user->{config}->{offset_sec} || 0;

    $c = $submissions->count(year => $year,
                             month => $month,
                             day => $day,
                             offset_sec => $offset);
    ok($c, "get count of the day");
    ok($c > 0, "valid count of the day");

    $c = $submissions->count(year => $year,
                             month => $month,
                             offset_sec => $offset);
    ok($c, "get count of the month");
    ok($c->{$day}, "valid count of the month");

    $c = $submissions->count(year => $year,
                             offset_sec => $offset);
    ok($c, "get count of the year");
    ok($c->{$month}, "valid count of the year");

    ok($submissions->update(submission_id => $subid, del => 1), "update succeed");
    $subm = $submissions->select(submission_id => $subid);
    ok($subm, "select story by submission_id (2)");
    is($subm->{del}, 1, "update valid");

    ok($submissions->update(submission_id => $subid, public => "no"), "update succeed (2)");
    $subm = $submissions->select(submission_id => $subid, show_nonpublic => 1);
    ok($subm, "select story by submission_id (3)");
    is($subm->{public}, "no", "update valid (2)");

    ok($submissions->hard_delete(submission_id => $subid), "delete story");
    $subm = $submissions->select(submission_id => $subid);
    ok(!$subm, "submission delete");
};


$test_man->cleanup;

done_testing();
