# -*-Perl-*-
# Newslash::Model:Tags related tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::Test qw(create_user delete_user
                            create_story delete_story
                            create_comments
                            create_users create_admin_user
                            get_anonymous_user
                          );
use List::Util qw(any);

my $t = Test::Mojo->new('Newslash::Web');
my ($user, $admin, $story, $globjid);

my $tags = $t->app->model('tags');
ok($tags, "get Tags instance");

# prepare for test
if ($t->app->mode eq 'test') {
    # at first, create test users.
    $user = create_user($t->app, "tagstest");
    ok($user, "create user");
    $admin = create_admin_user($t->app, "tagtest_admin");
    ok($admin, "create admin user");

    # second, create story.
    $story = create_story($t->app, $admin, "タグテスト", "<p>タグテスト本文</p>");
    ok($story, "create story for test");
    $globjid = $t->app->model('globjs')->get_globjid_from_item(story => $story->{stoid});
    ok($globjid, "get globjid of story");
}

subtest 'get topics' => sub {
    my $topics = $tags->get_topics;
    ok($topics, "get topics");
};

subtest 'set and get tag' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');


    my $params = {
                 };
    my $tagid = $tags->set_tag(globj_id => $globjid,
                               uid => $admin->{uid},
                               name => "slash");
    ok($tagid, "set tag");

    my $tag = $tags->select_tagnames(tagname => "slash");
    ok($tag, "select from tagnames");
    is($tag->{tagname}, "slash", "retrive valid tagname");

    my $rs_tags = $tags->select(globjid => $globjid);
    ok($rs_tags, "select tags");
    ok(any { $_->{tagid} == $tagid } @$rs_tags, "select by globjid");

    $rs_tags = $tags->select_usertag_for_globjids($admin->{uid}, $globjid);
    ok($rs_tags, "select_usertag");
    ok(any { $_->{tagid} == $tagid } @$rs_tags, "select usertag by globjid");
};


subtest 'set/add/del' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $tagid = $tags->add(globj_id => $globjid,
                           uid => $admin->{uid},
                           name => "hogehoge");
    ok($tagid, "add tag");
    my $rs_tags = $tags->select(globjid => $globjid);
    ok($rs_tags, "select tags");
    ok(any { $_->{tagid} == $tagid } @$rs_tags, "select by globjid");


    my $test_tag = crypt(rand, "hoge");
    $test_tag =~ s/\W//;
    $test_tag = "a$test_tag";

    my $rs = $tags->set(globj_id => $globjid,
                        uid => $admin->{uid},
                        names => ["foo", "bar", $test_tag]);
    ok($rs, "set tags");
    $rs_tags = $tags->select(globjid => $globjid);
    ok($rs_tags, "select tags 2");
    #warn Dumper $rs_tags;
    is(@$rs_tags, 3, "set valid number of tags");
    my @result = sort { $a->{tagname} cmp $b->{tagname} } @$rs_tags;
    is($result[0]->{tagname}, $test_tag, "valid tag 1");
    is($result[1]->{tagname}, "bar", "valid tag 2");
    is($result[2]->{tagname}, "foo", "valid tag 3");

    $rs = $tags->del(globj_id => $globjid,
                        uid => $admin->{uid},
                        name => $test_tag);
    ok($rs, "del tag");
    $rs_tags = $tags->select(globjid => $globjid);
    ok($rs_tags, "select tags 3");
    is(@$rs_tags, 2, "set valid number of tags 2");
    @result = sort { $a->{tagname} cmp $b->{tagname} } @$rs_tags;
    is($result[0]->{tagname}, "bar", "valid tag 4");
    is($result[1]->{tagname}, "foo", "valid tag 5");

};


# cleanup
if ($t->app->mode eq 'test') {
    delete_user($t->app, $user->{nickname});
    delete_user($t->app, $admin->{nickname});
    delete_story($t->app, $story->{stoid});
}

done_testing();
