# -*-Perl-*-
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;

use IO::Compress::Gzip qw(gzip);
use IO::Uncompress::Gunzip qw(gunzip);

my $t = Test::Mojo->new('Newslash::Web');
my $app = $t->app;


subtest 'CompositeCache test' => sub {

    my $ccache = $app->ccache;
    ok($ccache, "CompositeCache enabled");
    return if !$ccache;

    # simple cache
    my $testvalue = "testvalue";
    ok($ccache->set(".testkey", 1, $testvalue), "cache testvalue");
    is($ccache->get(".testkey", 1), $testvalue, "get testvalue");
    ok($ccache->del(".testkey", 1), "delete cache");

    my $testvalue2 = { foo => "bar", hoge => 1 };
    ok($ccache->set(".testkey", 2, $testvalue2), "cache hashvalue");
    is_deeply($ccache->get(".testkey", 2), $testvalue2, "get hashvalue");
    ok($ccache->del(".testkey", 2), "delete cache");

    # test model cache
    my $users = $app->model('users');
    my $test_user = $users->select(uid => 1);
    ok($test_user, "retrieve test user");

    my $cached_model = $ccache->model('users');
    my $rs = $cached_model->select(uid => 1);
    is_deeply($test_user, $rs, "non-cached read");

    $rs = $cached_model->select(uid => 1);
    is_deeply($test_user, $rs, "cached read");

    # disable immemory cache
    delete $ccache->{inmemory};

    # test hash cache
    my $testvalue3 = { foo => "bar", hoge => 1 };
    ok($ccache->set(".testkey", 3, $testvalue3), "kvs cache hashvalue");
    is_deeply($ccache->get(".testkey", 3), $testvalue3, "kvs cache get hashvalue");
    ok($ccache->del(".testkey", 3), "kcs cache delete");

    # binary data caching
    my $test_text = "あいうえおかきくけこ";
    my $test_text_bin = Encode::encode_utf8($test_text);
    gzip(\$test_text_bin, \my $compressed);
    my $testvalue4 = { foo => "bar", gzipped => $compressed };
    ok($ccache->hash_set(".testkey", 4, $testvalue4), "kvs cache binary value");
    is_deeply($ccache->hash_get(".testkey", 4), $testvalue4, "kvs get binary value");

    my $gunzipped;
    my $c = $ccache->hash_get(".testkey", 4)->{gzipped};
    is($c, $compressed, "get gzipped item");

    gunzip(\$compressed, \$gunzipped);
    is($gunzipped, $test_text_bin, "gunzip succeeded (not cached item)");
    is(Encode::decode_utf8($gunzipped), $test_text, "gunzipped item is valid text");

    gunzip(\$c, \$gunzipped);
    is($gunzipped, $test_text_bin, "gunzip succeeded (cached item)");
    is(Encode::decode_utf8($gunzipped), $test_text, "gunzipped item is valid text");

    ok($ccache->del(".testkey", 4), "delete kvs cache");

};

done_testing;
