#!/usr/bin/python
import time
import datetime
import calendar

import newslash_db
import lucene_wrapper
from htmlutil import strip_html_tag

from yaml import load
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

CONFIG_FILE="./.config.yml"

def main():
    config = load_config(CONFIG_FILE)
    db_cfg = config["Database"]

    stories = newslash_db.Stories(db_cfg)
    items = stories.select(limit=1000)

    lucene_wrapper.init_vm()
    
    start_time = time.time()
    with lucene_wrapper.Indexer(index_directory="./lucene_index") as indexer:
        for item in items:
            doc = lucene_wrapper.Document()
            if item["time"] is None:
                continue

            timestamp = calendar.timegm(item["time"].utctimetuple())
            (content_text, urls) = strip_html_tag(item["introtext"] + item["bodytext"])

            doc.add_string_field("type", "story")
            doc.add_string_field("id", item["sid"])

            doc.add_text_field("title", item["title"])
            doc.add_text_field("content_text", content_text)
            doc.add_text_field("dept", item["dept"])

            doc.add_int_field("create_time", timestamp)
            doc.add_int_field("topic", item["tid"])
            doc.add_int_field("author", item["uid"])
            doc.add_int_field("submitter", item["submitter"])

            for url in urls:
                doc.add_string_field("url", url)

            print("index {}...".format(item["sid"]))
            indexer.add(doc)
    print("indexing done. total time: {}s".format(time.time() - start_time))

def load_config(pathname):
    fh = open(pathname)
    config = load(fh, Loader=Loader)
    fh.close()
    return config


if __name__ == "__main__":
    main()
