#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use feature ':5.10';

use Getopt::Std;
use DBI;
use JSON qw(to_json);
use IO::File;
use File::Basename;
use File::Spec;

my $PROGNAME = basename($0);
my $VERSION = '0.1.0';

sub main {
    my $opts = {};

    if (!getopts( "hvru:H:p:d:", $opts)) {
        usage('Options used incorrectly.');
    }

    if ($opts->{'h'}) {
        usage();
    }
    my $output_file = shift @ARGV;
    usage() if !$output_file;

    my $dbh = connect_db($opts);
    if (!$dbh) {
        usage('cannot connect to database.');
    }

    # read from database
    my $datas = read_from_db($dbh);
    $dbh->disconnect;

    # convert json data
    my $output = {};
    $output->{items} = [];
    for my $item (@$datas) {
        my $i = {};
        $i->{id} = $item->{code} + 0;
        $i->{name} = "";
        $i->{seclev} = $item->{seclev} + 0;
        $i->{description} = $item->{type};
        $i->{selectable} = 1;
        push @{$output->{items}}, $i;
    }
    # output JSON
    my $json = JSON->new->pretty;
    my $json_text = $json->encode($output);
    my $fh = IO::File->new($output_file, "w");
    if (!$fh) {
        die "cannot write to file. exit.";
    }
    print $fh $json_text;
    $fh->close;
}

sub read_from_db {
    my $dbh = shift;

    my $sql = <<"EOSQL";
SELECT * FROM message_codes ORDER BY code ASC;
EOSQL
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $rs = $sth->fetchall_arrayref({});
    if (!defined $rs) {
        warn "database select error";
        exit 255;
    }
    $dbh->disconnect;
    return $rs;
}


sub connect_db {
  my $opts = shift @_;

  my $host = $opts->{'H'} || "";
  my $user = $opts->{'u'} || "";
  my $password = $opts->{'p'} || "";
  my $name = $opts->{'d'} || "";

  my $dbh = DBI->connect("DBI:mysql:$name:$host", $user, $password);
  return $dbh;
}

sub usage {
  print "*** $_[0] \n" if $_[0];
  print <<EOT;

Usage: $PROGNAME [OPTIONS] <output file>

This will read datas from message_codes table and save to file.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
EOT
  exit;
}

sub version {
  print <<EOT;


$PROGNAME $VERSION
EOT
  exit;
}

main();

__END__
